# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import base64
import time

from azure.core.exceptions import ServiceRequestError
from azure.core.pipeline.policies import HTTPPolicy


class AAZBearerTokenCredentialPolicy(HTTPPolicy):
    """

    Followed the implement of azure.core.pipeline.policies.BearerTokenCredentialPolicy
    """

    def __init__(self, credential, *scopes, **kwargs):  # pylint:disable=unused-argument
        super().__init__()
        self._scopes = scopes
        self._credential = credential
        self._token = None
        self._aux_tokens = None

    @staticmethod
    def _enforce_https(request):
        # move 'enforce_https' from options to context so it persists
        # across retries but isn't passed to a transport implementation
        option = request.context.options.pop("enforce_https", None)

        # True is the default setting; we needn't preserve an explicit opt in to the default behavior
        if option is False:
            request.context["enforce_https"] = option

        enforce_https = request.context.get("enforce_https", True)
        if enforce_https and not request.http_request.url.lower().startswith("https"):
            raise ServiceRequestError(
                "Bearer token authentication is not permitted for non-TLS protected (non-https) URLs."
            )

    def _update_headers(self, headers):
        """Updates the Authorization header with the bearer token.

        :param dict headers: The HTTP Request headers
        """

        headers["Authorization"] = "Bearer {}".format(self._token.token)
        if self._aux_tokens:
            headers["x-ms-authorization-auxiliary"] = ', '.join(
                "Bearer {}".format(token.token) for token in self._aux_tokens)

    @property
    def _need_new_token(self):
        if not self._token:
            return True
        return self._token.expires_on - time.time() < 300

    @property
    def _need_new_aux_tokens(self):
        if not hasattr(self._credential, 'get_auxiliary_tokens'):
            return False
        if not self._aux_tokens:
            return True
        for token in self._aux_tokens:
            if token.expires_on - time.time() < 300:
                return True
        return False

    def on_request(self, request):
        """Called before the policy sends a request.

        The base implementation authorizes the request with a bearer token.

        :param ~azure.core.pipeline.PipelineRequest request: the request
        """
        self._enforce_https(request)

        if self._need_new_token:
            self._token = self._credential.get_token(*self._scopes)

        if self._need_new_aux_tokens:
            self._aux_tokens = self._credential.get_auxiliary_tokens(*self._scopes)

        self._update_headers(request.http_request.headers)

    def authorize_request(self, request, *scopes, **kwargs):
        """Acquire a token from the credential and authorize the request with it.

        Keyword arguments are passed to the credential's get_token method. The token will be cached and used to
        authorize future requests.

        :param ~azure.core.pipeline.PipelineRequest request: the request
        :param str scopes: required scopes of authentication
        """
        if self._need_new_token:
            self._token = self._credential.get_token(*scopes, **kwargs)

        if self._need_new_aux_tokens:
            self._aux_tokens = self._credential.get_auxiliary_tokens(*self._scopes)

        self._update_headers(request.http_request.headers)

    def send(self, request):
        """Authorize request with a bearer token and send it to the next policy

        :param request: The pipeline request object
        :type request: ~azure.core.pipeline.PipelineRequest
        """
        self.on_request(request)
        try:
            response = self.next.send(request)
            self.on_response(request, response)
        except Exception:  # pylint:disable=broad-except
            handled = self.on_exception(request)
            if not handled:
                raise
        else:
            if response.http_response.status_code == 401:
                self._token = None  # any cached token is invalid
                if "WWW-Authenticate" in response.http_response.headers:
                    request_authorized = self.on_challenge(request, response)
                    if request_authorized:
                        try:
                            response = self.next.send(request)
                            self.on_response(request, response)
                        except Exception:  # pylint:disable=broad-except
                            handled = self.on_exception(request)
                            if not handled:
                                raise

        return response

    def on_challenge(self, request, response):
        """Authorize request according to an authentication challenge

        This method is called when the resource provider responds 401 with a WWW-Authenticate header.

        :param ~azure.core.pipeline.PipelineRequest request: the request which elicited an authentication challenge
        :param ~azure.core.pipeline.PipelineResponse response: the resource provider's response
        :returns: a bool indicating whether the policy should send the request
        """
        # pylint:disable=unused-argument,no-self-use
        return False

    def on_response(self, request, response):
        """Executed after the request comes back from the next policy.

        :param request: Request to be modified after returning from the policy.
        :type request: ~azure.core.pipeline.PipelineRequest
        :param response: Pipeline response object
        :type response: ~azure.core.pipeline.PipelineResponse
        """

    def on_exception(self, request):
        """Executed when an exception is raised while executing the next policy.

        This method is executed inside the exception handler.

        :param request: The Pipeline request object
        :type request: ~azure.core.pipeline.PipelineRequest
        :return: False by default, override with True to stop the exception.
        :rtype: bool
        """
        # pylint: disable=no-self-use,unused-argument
        return False


class AAZARMChallengeAuthenticationPolicy(AAZBearerTokenCredentialPolicy):
    """Adds a bearer token Authorization header to requests.

    This policy internally handles Continuous Access Evaluation (CAE) challenges. When it can't complete a challenge,
    it will return the 401 (unauthorized) response from ARM.

    :param ~azure.core.credentials.TokenCredential credential: credential for authorizing requests
    :param str scopes: required authentication scopes
    """

    @staticmethod
    def _parse_claims_challenge(challenge):
        """Parse the "claims" parameter from an authentication challenge

        Example challenge with claims:
            Bearer authorization_uri="https://login.windows-ppe.net/", error="invalid_token",
            error_description="User session has been revoked",
            claims="eyJhY2Nlc3NfdG9rZW4iOnsibmJmIjp7ImVzc2VudGlhbCI6dHJ1ZSwgInZhbHVlIjoiMTYwMzc0MjgwMCJ9fX0="

        :return: the challenge's "claims" parameter or None, if it doesn't contain that parameter
        """
        encoded_claims = None
        for parameter in challenge.split(","):
            if "claims=" in parameter:
                if encoded_claims:
                    # multiple claims challenges, e.g. for cross-tenant auth, would require special handling
                    return None
                encoded_claims = parameter[parameter.index("=") + 1:].strip(" \"'")

        if not encoded_claims:
            return None

        padding_needed = -len(encoded_claims) % 4
        try:
            decoded_claims = base64.urlsafe_b64decode(encoded_claims + "=" * padding_needed).decode()
            return decoded_claims
        except Exception:  # pylint:disable=broad-except
            return None

    def on_challenge(self, request, response):  # pylint:disable=unused-argument
        """Authorize request according to an ARM authentication challenge

        :param ~azure.core.pipeline.PipelineRequest request: the request which elicited an authentication challenge
        :param ~azure.core.pipeline.PipelineResponse response: ARM's response
        :returns: a bool indicating whether the policy should send the request
        """

        challenge = response.http_response.headers.get("WWW-Authenticate")
        if challenge:
            claims = self._parse_claims_challenge(challenge)
            if claims:
                self.authorize_request(request, *self._scopes, claims=claims)
                return True

        return False



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "afd origin-group create",
)
class Create(AAZCommand):
    """Create a new origin group within the specified profile.

    :example: Creates a new origin group within the specified profile.
        az afd origin-group create --resource-group RG --profile-name profile1 --origin-group-name origingroup1 --sample-size 3 --successful-samples-required 3 --additional-latency-in-milliseconds 1000 --health-probe-settings "{probe-path:/path2,probe-request-type:NotSet,probe-protocol:NotSet,probe-interval-in-seconds:10}" --traffic-restoration-time-to-healed-or-new-endpoints-in-minutes 5
    """

    _aaz_info = {
        "version": "2025-06-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.cdn/profiles/{}/origingroups/{}", "2025-06-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.origin_group_name = AAZStrArg(
            options=["-n", "--name", "--origin-group-name"],
            help="Name of the origin group which is unique within the endpoint.",
            required=True,
        )
        _args_schema.profile_name = AAZStrArg(
            options=["--profile-name"],
            help="Name of the Azure Front Door Standard or Azure Front Door Premium which is unique within the resource group.",
            required=True,
            fmt=AAZStrArgFormat(
                pattern="^[a-zA-Z0-9]+(-*[a-zA-Z0-9])*$",
                max_length=260,
                min_length=1,
            ),
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "LoadBalancingSettings"

        _args_schema = cls._args_schema
        _args_schema.additional_latency_in_milliseconds = AAZIntArg(
            options=["--additional-latency-in-milliseconds"],
            arg_group="LoadBalancingSettings",
            help="The additional latency in milliseconds for probes to fall into the lowest latency bucket",
        )
        _args_schema.sample_size = AAZIntArg(
            options=["--sample-size"],
            arg_group="LoadBalancingSettings",
            help="The number of samples to consider for load balancing decisions",
        )
        _args_schema.successful_samples_required = AAZIntArg(
            options=["--successful-samples-required"],
            arg_group="LoadBalancingSettings",
            help="The number of samples within the sample period that must succeed",
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.authentication = AAZObjectArg(
            options=["--authentication"],
            arg_group="Properties",
            help="Authentication settings for origin in origin group.",
        )
        _args_schema.health_probe_settings = AAZObjectArg(
            options=["--health-probe-settings"],
            arg_group="Properties",
            help="Health probe settings to the origin that is used to determine the health of the origin.",
        )
        _args_schema.session_affinity_state = AAZStrArg(
            options=["--session-affinity-state"],
            arg_group="Properties",
            help="Whether to allow session affinity on this host. Valid options are 'Enabled' or 'Disabled'",
            enum={"Disabled": "Disabled", "Enabled": "Enabled"},
        )
        _args_schema.traffic_restoration_time_to_healed_or_new_endpoints_in_minutes = AAZIntArg(
            options=["--traffic-restoration-time-to-healed-or-new-endpoints-in-minutes"],
            arg_group="Properties",
            help="Time in minutes to shift the traffic to the endpoint gradually when an unhealthy endpoint comes healthy or a new endpoint is added. Default is 10 mins. This property is currently not supported.",
            fmt=AAZIntArgFormat(
                maximum=50,
                minimum=0,
            ),
        )

        authentication = cls._args_schema.authentication
        authentication.scope = AAZStrArg(
            options=["scope"],
            help="The scope used when requesting token from Microsoft Entra. For example, for Azure Blob Storage, scope could be \"https://storage.azure.com/.default\".",
        )
        authentication.type = AAZStrArg(
            options=["type"],
            help="The type of the authentication for the origin.",
            enum={"SystemAssignedIdentity": "SystemAssignedIdentity", "UserAssignedIdentity": "UserAssignedIdentity"},
        )
        authentication.user_assigned_identity = AAZObjectArg(
            options=["user-assigned-identity"],
            help="The user assigned managed identity to use for the origin authentication if type is UserAssignedIdentity.",
        )

        user_assigned_identity = cls._args_schema.authentication.user_assigned_identity
        user_assigned_identity.id = AAZStrArg(
            options=["id"],
            help="Resource ID.",
        )

        health_probe_settings = cls._args_schema.health_probe_settings
        health_probe_settings.probe_interval_in_seconds = AAZIntArg(
            options=["probe-interval-in-seconds"],
            help="The number of seconds between health probes.Default is 240sec.",
            fmt=AAZIntArgFormat(
                maximum=255,
                minimum=1,
            ),
        )
        health_probe_settings.probe_path = AAZStrArg(
            options=["probe-path"],
            help="The path relative to the origin that is used to determine the health of the origin.",
        )
        health_probe_settings.probe_protocol = AAZStrArg(
            options=["probe-protocol"],
            help="Protocol to use for health probe.",
            enum={"Http": "Http", "Https": "Https", "NotSet": "NotSet"},
        )
        health_probe_settings.probe_request_type = AAZStrArg(
            options=["probe-request-type"],
            help="The type of health probe request that is made.",
            enum={"GET": "GET", "HEAD": "HEAD", "NotSet": "NotSet"},
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.AFDOriginGroupsCreate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class AFDOriginGroupsCreate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Cdn/profiles/{profileName}/originGroups/{originGroupName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "originGroupName", self.ctx.args.origin_group_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "profileName", self.ctx.args.profile_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-06-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("authentication", AAZObjectType, ".authentication")
                properties.set_prop("healthProbeSettings", AAZObjectType, ".health_probe_settings")
                properties.set_prop("loadBalancingSettings", AAZObjectType)
                properties.set_prop("sessionAffinityState", AAZStrType, ".session_affinity_state")
                properties.set_prop("trafficRestorationTimeToHealedOrNewEndpointsInMinutes", AAZIntType, ".traffic_restoration_time_to_healed_or_new_endpoints_in_minutes")

            authentication = _builder.get(".properties.authentication")
            if authentication is not None:
                authentication.set_prop("scope", AAZStrType, ".scope")
                authentication.set_prop("type", AAZStrType, ".type")
                authentication.set_prop("userAssignedIdentity", AAZObjectType, ".user_assigned_identity")

            user_assigned_identity = _builder.get(".properties.authentication.userAssignedIdentity")
            if user_assigned_identity is not None:
                user_assigned_identity.set_prop("id", AAZStrType, ".id")

            health_probe_settings = _builder.get(".properties.healthProbeSettings")
            if health_probe_settings is not None:
                health_probe_settings.set_prop("probeIntervalInSeconds", AAZIntType, ".probe_interval_in_seconds")
                health_probe_settings.set_prop("probePath", AAZStrType, ".probe_path")
                health_probe_settings.set_prop("probeProtocol", AAZStrType, ".probe_protocol")
                health_probe_settings.set_prop("probeRequestType", AAZStrType, ".probe_request_type")

            load_balancing_settings = _builder.get(".properties.loadBalancingSettings")
            if load_balancing_settings is not None:
                load_balancing_settings.set_prop("additionalLatencyInMilliseconds", AAZIntType, ".additional_latency_in_milliseconds")
                load_balancing_settings.set_prop("sampleSize", AAZIntType, ".sample_size")
                load_balancing_settings.set_prop("successfulSamplesRequired", AAZIntType, ".successful_samples_required")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _CreateHelper._build_schema_afd_origin_group_read(cls._schema_on_200_201)

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""

    _schema_afd_origin_group_read = None

    @classmethod
    def _build_schema_afd_origin_group_read(cls, _schema):
        if cls._schema_afd_origin_group_read is not None:
            _schema.id = cls._schema_afd_origin_group_read.id
            _schema.name = cls._schema_afd_origin_group_read.name
            _schema.properties = cls._schema_afd_origin_group_read.properties
            _schema.system_data = cls._schema_afd_origin_group_read.system_data
            _schema.type = cls._schema_afd_origin_group_read.type
            return

        cls._schema_afd_origin_group_read = _schema_afd_origin_group_read = AAZObjectType()

        afd_origin_group_read = _schema_afd_origin_group_read
        afd_origin_group_read.id = AAZStrType(
            flags={"read_only": True},
        )
        afd_origin_group_read.name = AAZStrType(
            flags={"read_only": True},
        )
        afd_origin_group_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        afd_origin_group_read.system_data = AAZObjectType(
            serialized_name="systemData",
            flags={"read_only": True},
        )
        afd_origin_group_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_afd_origin_group_read.properties
        properties.authentication = AAZObjectType()
        properties.deployment_status = AAZStrType(
            serialized_name="deploymentStatus",
            flags={"read_only": True},
        )
        properties.health_probe_settings = AAZObjectType(
            serialized_name="healthProbeSettings",
        )
        properties.load_balancing_settings = AAZObjectType(
            serialized_name="loadBalancingSettings",
        )
        properties.profile_name = AAZStrType(
            serialized_name="profileName",
            flags={"read_only": True},
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.session_affinity_state = AAZStrType(
            serialized_name="sessionAffinityState",
        )
        properties.traffic_restoration_time_to_healed_or_new_endpoints_in_minutes = AAZIntType(
            serialized_name="trafficRestorationTimeToHealedOrNewEndpointsInMinutes",
        )

        authentication = _schema_afd_origin_group_read.properties.authentication
        authentication.scope = AAZStrType()
        authentication.type = AAZStrType()
        authentication.user_assigned_identity = AAZObjectType(
            serialized_name="userAssignedIdentity",
        )

        user_assigned_identity = _schema_afd_origin_group_read.properties.authentication.user_assigned_identity
        user_assigned_identity.id = AAZStrType()

        health_probe_settings = _schema_afd_origin_group_read.properties.health_probe_settings
        health_probe_settings.probe_interval_in_seconds = AAZIntType(
            serialized_name="probeIntervalInSeconds",
        )
        health_probe_settings.probe_path = AAZStrType(
            serialized_name="probePath",
        )
        health_probe_settings.probe_protocol = AAZStrType(
            serialized_name="probeProtocol",
        )
        health_probe_settings.probe_request_type = AAZStrType(
            serialized_name="probeRequestType",
        )

        load_balancing_settings = _schema_afd_origin_group_read.properties.load_balancing_settings
        load_balancing_settings.additional_latency_in_milliseconds = AAZIntType(
            serialized_name="additionalLatencyInMilliseconds",
        )
        load_balancing_settings.sample_size = AAZIntType(
            serialized_name="sampleSize",
        )
        load_balancing_settings.successful_samples_required = AAZIntType(
            serialized_name="successfulSamplesRequired",
        )

        system_data = _schema_afd_origin_group_read.system_data
        system_data.created_at = AAZStrType(
            serialized_name="createdAt",
        )
        system_data.created_by = AAZStrType(
            serialized_name="createdBy",
        )
        system_data.created_by_type = AAZStrType(
            serialized_name="createdByType",
        )
        system_data.last_modified_at = AAZStrType(
            serialized_name="lastModifiedAt",
        )
        system_data.last_modified_by = AAZStrType(
            serialized_name="lastModifiedBy",
        )
        system_data.last_modified_by_type = AAZStrType(
            serialized_name="lastModifiedByType",
        )

        _schema.id = cls._schema_afd_origin_group_read.id
        _schema.name = cls._schema_afd_origin_group_read.name
        _schema.properties = cls._schema_afd_origin_group_read.properties
        _schema.system_data = cls._schema_afd_origin_group_read.system_data
        _schema.type = cls._schema_afd_origin_group_read.type


__all__ = ["Create"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.command_modules.dms.validators import throw_if_not_dictionary, throw_if_not_list
from azure.cli.core.azclierror import ValidationError
from azure.mgmt.datamigration.models import (MigrateSqlServerSqlDbTaskInput,
                                             MigrateSqlServerSqlDbDatabaseInput,
                                             MigrationValidationOptions,
                                             MigratePostgreSqlAzureDbForPostgreSqlSyncTaskInput,
                                             MigratePostgreSqlAzureDbForPostgreSqlSyncDatabaseInput,
                                             MigratePostgreSqlAzureDbForPostgreSqlSyncDatabaseTableInput,
                                             MigrateMySqlAzureDbForMySqlOfflineTaskInput,
                                             MigrateMySqlAzureDbForMySqlOfflineDatabaseInput)


def get_migrate_sql_to_sqldb_offline_input(database_options_json,
                                           source_connection_info,
                                           target_connection_info,
                                           enable_schema_validation,
                                           enable_data_integrity_validation,
                                           enable_query_analysis_validation):
    database_options = []

    for d in database_options_json:
        database_options.append(
            MigrateSqlServerSqlDbDatabaseInput(
                name=d.get('name', None),
                target_database_name=d.get('target_database_name', None),
                make_source_db_read_only=d.get('make_source_db_read_only', None),
                table_map=d.get('table_map', None)))

    validation_options = MigrationValidationOptions(enable_schema_validation=enable_schema_validation,
                                                    enable_data_integrity_validation=enable_data_integrity_validation,
                                                    enable_query_analysis_validation=enable_query_analysis_validation)

    return MigrateSqlServerSqlDbTaskInput(source_connection_info=source_connection_info,
                                          target_connection_info=target_connection_info,
                                          selected_databases=database_options,
                                          validation_options=validation_options)


def get_migrate_postgresql_to_azuredbforpostgresql_sync_input(database_options_json,
                                                              source_connection_info,
                                                              target_connection_info):
    database_options = []

    for d in database_options_json:
        s_t = d.get('selectedTables', None)
        t = None if s_t is None else [MigratePostgreSqlAzureDbForPostgreSqlSyncDatabaseTableInput(name=t) for t in s_t]
        database_options.append(
            MigratePostgreSqlAzureDbForPostgreSqlSyncDatabaseInput(
                name=d.get('name', None),
                target_database_name=d.get('target_database_name', None),
                migration_setting=d.get('migrationSetting', None),
                source_setting=d.get('sourceSetting', None),
                target_setting=d.get('targetSetting', None),
                selected_tables=t))

    return MigratePostgreSqlAzureDbForPostgreSqlSyncTaskInput(source_connection_info=source_connection_info,
                                                              target_connection_info=target_connection_info,
                                                              selected_databases=database_options)


def get_migrate_mysql_to_azuredbformysql_sync_input(database_options_json,
                                                    source_connection_info,
                                                    target_connection_info):
    return get_migrate_mysql_to_azuredbformysql_input(database_options_json,
                                                      source_connection_info,
                                                      target_connection_info,
                                                      has_schema_migration_options=True,
                                                      has_consistent_snapshot_options=True,
                                                      has_binlog_position=False)


def get_migrate_mysql_to_azuredbformysql_offline_input(database_options_json,
                                                       source_connection_info,
                                                       target_connection_info):
    return get_migrate_mysql_to_azuredbformysql_input(database_options_json,
                                                      source_connection_info,
                                                      target_connection_info,
                                                      has_schema_migration_options=True,
                                                      has_consistent_snapshot_options=True,
                                                      has_binlog_position=False)


def get_migrate_mysql_to_azuredbformysql_cdc_input(database_options_json,
                                                   source_connection_info,
                                                   target_connection_info):
    return get_migrate_mysql_to_azuredbformysql_input(database_options_json,
                                                      source_connection_info,
                                                      target_connection_info,
                                                      has_schema_migration_options=False,
                                                      has_consistent_snapshot_options=False,
                                                      has_binlog_position=True)


def get_migrate_mysql_to_azuredbformysql_input(database_options_json,
                                               source_connection_info,
                                               target_connection_info,
                                               has_schema_migration_options: bool,
                                               has_consistent_snapshot_options: bool,
                                               has_binlog_position: bool):
    database_options = []
    migration_level_settings = {}
    make_source_server_read_only = False
    migration_properties = {}

    if not isinstance(database_options_json, dict):
        raise ValidationError('Format of the database option file is wrong')

    if 'selected_databases' not in database_options_json:
        raise ValidationError('Database option file should contain at least one selected database for migration')

    selected_databases = database_options_json.get('selected_databases')

    for database in selected_databases:
        if not isinstance(database, dict):
            raise ValidationError('Format of the selected database file is wrong')
        if 'name' not in database:
            raise ValidationError('Selected database should have a name')
        if 'target_database_name' not in database:
            raise ValidationError('Selected database should have a target_database_name')
        if 'table_map' in database and (not isinstance(database.get('table_map'), dict) or
                                        len(database.get('table_map')) == 0):
            raise ValidationError('table_map should be dictionary and non empty, to select all tables remove table_map')

        db_input = create_db_input(database, has_schema_migration_options)
        database_options.append(db_input)

    set_optional(migration_properties, 'sourceServerResourceId', database_options_json, 'source_server_resource_id')
    set_optional(migration_properties, 'targetServerResourceId', database_options_json, 'target_server_resource_id')

    if 'migration_level_settings' in database_options_json:
        migration_level_settings = database_options_json.get('migration_level_settings')
        if not isinstance(migration_level_settings, dict):
            raise ValidationError('migration_level_settings should be a dictionary')

    if has_consistent_snapshot_options:
        make_source_server_read_only = database_options_json.get('make_source_server_read_only', False)
        set_optional(migration_level_settings, 'enableConsistentBackup', database_options_json,
                     'enable_consistent_backup')
        set_optional(migration_level_settings, 'enableConsistentBackupWithoutLocks', database_options_json,
                     'enable_consistent_backup_without_locks')

    if has_schema_migration_options:
        extract_schema_migration_options(migration_properties, database_options_json)

    if has_binlog_position:
        set_required(migration_properties, 'binLogInfo', database_options_json, 'binlog_info', throw_if_not_dictionary)

    task_input = MigrateMySqlAzureDbForMySqlOfflineTaskInput(source_connection_info=source_connection_info,
                                                             target_connection_info=target_connection_info,
                                                             selected_databases=database_options,
                                                             optional_agent_settings=migration_level_settings,
                                                             make_source_server_read_only=make_source_server_read_only)

    if len(migration_properties) > 0:
        task_input.additional_properties = migration_properties
        task_input.enable_additional_properties_sending()

    return task_input


def extract_schema_migration_options(migration_properties, database_options_json):
    set_optional(migration_properties, 'migrateAllViews', database_options_json, 'migrate_all_views')
    set_optional(migration_properties, 'migrateAllTriggers', database_options_json, 'migrate_all_triggers')
    set_optional(migration_properties, 'migrateAllEvents', database_options_json, 'migrate_all_events')
    set_optional(migration_properties, 'migrateAllRoutines', database_options_json, 'migrate_all_routines')
    set_optional(migration_properties, 'migrateAllTablesSchema', database_options_json, 'migrate_all_tables_schema')
    set_optional(migration_properties, 'migrateUserSystemTables', database_options_json, 'migrate_user_system_tables')


def create_db_input(database, has_schema_migration_options):
    db_input = MigrateMySqlAzureDbForMySqlOfflineDatabaseInput(
        name=database.get('name'),
        target_database_name=database.get('target_database_name'),
        table_map=database.get('table_map'))

    if has_schema_migration_options:
        db_properties = {}
        set_optional(db_properties, 'tablesToMigrateSchema', database, 'tables_to_migrate_schema',
                     throw_if_not_dictionary)
        set_optional(db_properties, 'selectedViews', database, 'selected_views', throw_if_not_list)
        set_optional(db_properties, 'selectedTriggers', database, 'selected_triggers', throw_if_not_list)
        set_optional(db_properties, 'selectedRoutines', database, 'selected_routines', throw_if_not_list)
        set_optional(db_properties, 'selectedEvents', database, 'selected_events', throw_if_not_list)
        set_optional(db_properties,
                     'selectDatabaseForSchemaMigration',
                     database,
                     'select_database_for_schema_migration')

        if len(db_properties) > 0:
            db_input.additional_properties = db_properties
            db_input.enable_additional_properties_sending()

    return db_input


def set_optional(target: dict,
                 target_property: str,
                 source: dict,
                 source_property: str,
                 validator: callable([any, str]) = None):
    if source_property in source:
        value = source[source_property]
        if validator is not None:
            validator(value, source_property)
        target[target_property] = value


def set_required(target: dict,
                 target_property: str,
                 source: dict,
                 source_property: str,
                 validator: callable([any, str]) = None):
    if source_property in source:
        set_optional(target, target_property, source, source_property, validator)
    else:
        raise ValidationError("'%s' attribute is required but it is not found in the input json" % source_property)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def cf_connection_cl(cli_ctx, *_):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.servicelinker import ServiceLinkerManagementClient

    return get_mgmt_service_client(cli_ctx, ServiceLinkerManagementClient,
                                   subscription_bound=False, api_version="2024-07-01-preview")


def cf_linker(cli_ctx, *_):
    return cf_connection_cl(cli_ctx).linker


def cf_connector(cli_ctx, *_):
    return cf_connection_cl(cli_ctx).connector


def cf_configuration_names(cli_ctx, *_):
    return cf_connection_cl(cli_ctx).configuration_names



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "monitor activity-log alert create",
)
class Create(AAZCommand):
    """Create a default activity log alert rule.

    This command will create a default activity log with one condition which compares if the activities logs 'category' field equals to 'ServiceHealth'. The newly created activity log alert does not have any action groups attached to it.

    :example: Create an alert rule with default settings.
        az monitor activity-log alert create -n AlertName -g ResourceGroup

    :example: Create an alert rule with condition about error level service health log.
        az monitor activity-log alert create -n AlertName -g ResourceGroup --condition category=ServiceHealth and level=Error

    :example: Create an alert rule with an action group and specify webhook properties.
        az monitor activity-log alert create -n AlertName -g ResourceGroup -a /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/microsoft.insights/actionGroups/{ActionGroup} -w usage=test owner=jane

    :example: Create an alert rule which is initially disabled.
        az monitor activity-log alert create -n AlertName -g ResourceGroup --disable
    """

    _aaz_info = {
        "version": "2020-10-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.insights/activitylogalerts/{}", "2020-10-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.activity_log_alert_name = AAZStrArg(
            options=["-n", "--name", "--activity-log-alert-name"],
            help="The name of the activity log alert.",
            required=True,
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.location = AAZResourceLocationArg(
            help="The location of the resource. Since Azure Activity Log Alerts is a global service, the location of the rules should always be 'global'.",
            default="global",
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.action_groups = AAZListArg(
            options=["--action-groups"],
            help="The list of the Action Groups.",
        )
        _args_schema.all_of = AAZListArg(
            options=["--all-of"],
            help="The list of Activity Log Alert rule conditions.",
        )
        _args_schema.description = AAZStrArg(
            options=["--description"],
            help="A description of this Activity Log Alert rule.",
        )
        _args_schema.enabled = AAZBoolArg(
            options=["--enabled"],
            help="Indicates whether this Activity Log Alert rule is enabled. If an Activity Log Alert rule is not enabled, then none of its actions will be activated.",
            default=True,
        )
        _args_schema.scopes = AAZListArg(
            options=["--scopes"],
            help="A list of resource IDs that will be used as prefixes. The alert will only apply to Activity Log events with resource IDs that fall under one of these prefixes. This list must include at least one item.",
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            help="The tags of the resource.",
        )

        action_groups = cls._args_schema.action_groups
        action_groups.Element = AAZObjectArg()

        _element = cls._args_schema.action_groups.Element
        _element.action_group_id = AAZStrArg(
            options=["action-group-id"],
            help="The resource ID of the Action Group. This cannot be null or empty.",
            required=True,
        )
        _element.webhook_properties = AAZDictArg(
            options=["webhook-properties"],
            help="the dictionary of custom properties to include with the post operation. These data are appended to the webhook payload.",
        )

        webhook_properties = cls._args_schema.action_groups.Element.webhook_properties
        webhook_properties.Element = AAZStrArg()

        all_of = cls._args_schema.all_of
        all_of.Element = AAZObjectArg()

        _element = cls._args_schema.all_of.Element
        _element.any_of = AAZListArg(
            options=["any-of"],
            help="An Activity Log Alert rule condition that is met when at least one of its member leaf conditions are met.",
        )
        _element.contains_any = AAZListArg(
            options=["contains-any"],
            help="The value of the event's field will be compared to the values in this array (case-insensitive) to determine if the condition is met.",
        )
        _element.equals = AAZStrArg(
            options=["equals"],
            help="The value of the event's field will be compared to this value (case-insensitive) to determine if the condition is met.",
        )
        _element.field = AAZStrArg(
            options=["field"],
            help="The name of the Activity Log event's field that this condition will examine. The possible values for this field are (case-insensitive): 'resourceId', 'category', 'caller', 'level', 'operationName', 'resourceGroup', 'resourceProvider', 'status', 'subStatus', 'resourceType', or anything beginning with 'properties'.",
        )

        any_of = cls._args_schema.all_of.Element.any_of
        any_of.Element = AAZObjectArg()

        _element = cls._args_schema.all_of.Element.any_of.Element
        _element.contains_any = AAZListArg(
            options=["contains-any"],
            help="The value of the event's field will be compared to the values in this array (case-insensitive) to determine if the condition is met.",
        )
        _element.equals = AAZStrArg(
            options=["equals"],
            help="The value of the event's field will be compared to this value (case-insensitive) to determine if the condition is met.",
        )
        _element.field = AAZStrArg(
            options=["field"],
            help="The name of the Activity Log event's field that this condition will examine. The possible values for this field are (case-insensitive): 'resourceId', 'category', 'caller', 'level', 'operationName', 'resourceGroup', 'resourceProvider', 'status', 'subStatus', 'resourceType', or anything beginning with 'properties'.",
        )

        contains_any = cls._args_schema.all_of.Element.any_of.Element.contains_any
        contains_any.Element = AAZStrArg()

        contains_any = cls._args_schema.all_of.Element.contains_any
        contains_any.Element = AAZStrArg()

        scopes = cls._args_schema.scopes
        scopes.Element = AAZStrArg()

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg()
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.ActivityLogAlertsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class ActivityLogAlertsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200, 201]:
                return self.on_200_201(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/activityLogAlerts/{activityLogAlertName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "activityLogAlertName", self.ctx.args.activity_log_alert_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2020-10-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("location", AAZStrType, ".location")
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("actions", AAZObjectType, ".", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("condition", AAZObjectType, ".", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("description", AAZStrType, ".description")
                properties.set_prop("enabled", AAZBoolType, ".enabled")
                properties.set_prop("scopes", AAZListType, ".scopes", typ_kwargs={"flags": {"required": True}})

            actions = _builder.get(".properties.actions")
            if actions is not None:
                actions.set_prop("actionGroups", AAZListType, ".action_groups")

            action_groups = _builder.get(".properties.actions.actionGroups")
            if action_groups is not None:
                action_groups.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.actions.actionGroups[]")
            if _elements is not None:
                _elements.set_prop("actionGroupId", AAZStrType, ".action_group_id", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("webhookProperties", AAZDictType, ".webhook_properties")

            webhook_properties = _builder.get(".properties.actions.actionGroups[].webhookProperties")
            if webhook_properties is not None:
                webhook_properties.set_elements(AAZStrType, ".")

            condition = _builder.get(".properties.condition")
            if condition is not None:
                condition.set_prop("allOf", AAZListType, ".all_of", typ_kwargs={"flags": {"required": True}})

            all_of = _builder.get(".properties.condition.allOf")
            if all_of is not None:
                all_of.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.condition.allOf[]")
            if _elements is not None:
                _elements.set_prop("anyOf", AAZListType, ".any_of")
                _elements.set_prop("containsAny", AAZListType, ".contains_any")
                _elements.set_prop("equals", AAZStrType, ".equals")
                _elements.set_prop("field", AAZStrType, ".field")

            any_of = _builder.get(".properties.condition.allOf[].anyOf")
            if any_of is not None:
                any_of.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.condition.allOf[].anyOf[]")
            if _elements is not None:
                _elements.set_prop("containsAny", AAZListType, ".contains_any")
                _elements.set_prop("equals", AAZStrType, ".equals")
                _elements.set_prop("field", AAZStrType, ".field")

            contains_any = _builder.get(".properties.condition.allOf[].anyOf[].containsAny")
            if contains_any is not None:
                contains_any.set_elements(AAZStrType, ".")

            contains_any = _builder.get(".properties.condition.allOf[].containsAny")
            if contains_any is not None:
                contains_any.set_elements(AAZStrType, ".")

            scopes = _builder.get(".properties.scopes")
            if scopes is not None:
                scopes.set_elements(AAZStrType, ".")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.location = AAZStrType()
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.tags = AAZDictType()
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200_201.properties
            properties.actions = AAZObjectType(
                flags={"required": True},
            )
            properties.condition = AAZObjectType(
                flags={"required": True},
            )
            properties.description = AAZStrType()
            properties.enabled = AAZBoolType()
            properties.scopes = AAZListType(
                flags={"required": True},
            )

            actions = cls._schema_on_200_201.properties.actions
            actions.action_groups = AAZListType(
                serialized_name="actionGroups",
            )

            action_groups = cls._schema_on_200_201.properties.actions.action_groups
            action_groups.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.actions.action_groups.Element
            _element.action_group_id = AAZStrType(
                serialized_name="actionGroupId",
                flags={"required": True},
            )
            _element.webhook_properties = AAZDictType(
                serialized_name="webhookProperties",
            )

            webhook_properties = cls._schema_on_200_201.properties.actions.action_groups.Element.webhook_properties
            webhook_properties.Element = AAZStrType()

            condition = cls._schema_on_200_201.properties.condition
            condition.all_of = AAZListType(
                serialized_name="allOf",
                flags={"required": True},
            )

            all_of = cls._schema_on_200_201.properties.condition.all_of
            all_of.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.condition.all_of.Element
            _element.any_of = AAZListType(
                serialized_name="anyOf",
            )
            _element.contains_any = AAZListType(
                serialized_name="containsAny",
            )
            _element.equals = AAZStrType()
            _element.field = AAZStrType()

            any_of = cls._schema_on_200_201.properties.condition.all_of.Element.any_of
            any_of.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.condition.all_of.Element.any_of.Element
            _element.contains_any = AAZListType(
                serialized_name="containsAny",
            )
            _element.equals = AAZStrType()
            _element.field = AAZStrType()

            contains_any = cls._schema_on_200_201.properties.condition.all_of.Element.any_of.Element.contains_any
            contains_any.Element = AAZStrType()

            contains_any = cls._schema_on_200_201.properties.condition.all_of.Element.contains_any
            contains_any.Element = AAZStrType()

            scopes = cls._schema_on_200_201.properties.scopes
            scopes.Element = AAZStrType()

            tags = cls._schema_on_200_201.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""


__all__ = ["Create"]



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['network private-dns'] = """
type: group
short-summary: Manage Private DNS domains in Azure.
"""

helps['network private-dns record-set'] = """
type: group
short-summary: Manage Private DNS records and record sets.
"""

helps['network private-dns record-set a'] = """
type: group
short-summary: Manage Private DNS A records.
"""

helps['network private-dns record-set a add-record'] = """
type: command
short-summary: Add an A record.
examples:
  - name: Add an A record.
    text: |
        az network private-dns record-set a add-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -a MyIpv4Address
"""

helps['network private-dns record-set a create'] = """
type: command
short-summary: Create an empty A record set.
examples:
  - name: Create an empty A record set.
    text: az network private-dns record-set a create -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set a delete'] = """
type: command
short-summary: Delete an A record set and all associated records.
examples:
  - name: Delete an A record set and all associated records.
    text: az network private-dns record-set a delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set a list'] = """
type: command
short-summary: List all A record sets in a zone.
examples:
  - name: List all A record sets in a zone.
    text: az network private-dns record-set a list -g MyResourceGroup -z www.mysite.com
"""

helps['network private-dns record-set a remove-record'] = """
type: command
short-summary: Remove an A record from its record set.
long-summary: >
    By default, if the last record in a set is removed, the record set is deleted.
    To retain the empty record set, include --keep-empty-record-set.
examples:
  - name: Remove an A record from its record set.
    text: |
        az network private-dns record-set a remove-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -a MyIpv4Address
"""

helps['network private-dns record-set a show'] = """
type: command
short-summary: Get the details of an A record set.
examples:
  - name: Get the details of an A record set.
    text: az network private-dns record-set a show -g MyResourceGroup -n MyRecordSet -z www.mysite.com
"""

helps['network private-dns record-set a update'] = """
type: command
short-summary: Update an A record set.
examples:
  - name: Update an A record set.
    text: |
        az network private-dns record-set a update -g MyResourceGroup -n MyRecordSet \\
            -z www.mysite.com --metadata owner=WebTeam
  - name: Update an A record set. (autogenerated)
    text: |
        az network private-dns record-set a update --name MyRecordSet --resource-group MyResourceGroup --set useRemoteGateways=true --subscription MySubscription --zone-name www.mysite.com
    crafted: true
"""

helps['network private-dns record-set aaaa'] = """
type: group
short-summary: Manage Private DNS AAAA records.
"""

helps['network private-dns record-set aaaa add-record'] = """
type: command
short-summary: Add an AAAA record.
examples:
  - name: Add an AAAA record.
    text: |
        az network private-dns record-set aaaa add-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -a MyIpv6Address
"""

helps['network private-dns record-set aaaa create'] = """
type: command
short-summary: Create an empty AAAA record set.
examples:
  - name: Create an empty AAAA record set.
    text: az network private-dns record-set aaaa create -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set aaaa delete'] = """
type: command
short-summary: Delete an AAAA record set and all associated records.
examples:
  - name: Delete an AAAA record set and all associated records.
    text: az network private-dns record-set aaaa delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set aaaa list'] = """
type: command
short-summary: List all AAAA record sets in a zone.
examples:
  - name: List all AAAA record sets in a zone.
    text: az network private-dns record-set aaaa list -g MyResourceGroup -z www.mysite.com
"""

helps['network private-dns record-set aaaa remove-record'] = """
type: command
short-summary: Remove AAAA record from its record set.
long-summary: >
    By default, if the last record in a set is removed, the record set is deleted.
    To retain the empty record set, include --keep-empty-record-set.
examples:
  - name: Remove an AAAA record from its record set.
    text: |
        az network private-dns record-set aaaa remove-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -a MyIpv6Address
"""

helps['network private-dns record-set aaaa show'] = """
type: command
short-summary: Get the details of an AAAA record set.
examples:
  - name: Get the details of an AAAA record set.
    text: az network private-dns record-set aaaa show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set aaaa update'] = """
type: command
short-summary: Update an AAAA record set.
examples:
  - name: Update an AAAA record set.
    text: |
        az network private-dns record-set aaaa update -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet --metadata owner=WebTeam
"""

helps['network private-dns record-set cname'] = """
type: group
short-summary: Manage Private DNS CNAME records.
"""

helps['network private-dns record-set cname create'] = """
type: command
short-summary: Create an empty CNAME record set.
examples:
  - name: Create an empty CNAME record set.
    text: az network private-dns record-set cname create -g MyResourceGroup -z www.mysite.com -n MyRecordSet
  - name: Create an empty CNAME record set (autogenerated)
    text: |
        az network private-dns record-set cname create --name MyRecordSet --resource-group MyResourceGroup --subscription MySubscription --ttl 300 --zone-name www.mysite.com
    crafted: true
"""

helps['network private-dns record-set cname delete'] = """
type: command
short-summary: Delete a CNAME record set and its associated record.
examples:
  - name: Delete a CNAME record set and its associated record.
    text: az network private-dns record-set cname delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
  - name: Delete a CNAME record set and its associated record. (autogenerated)
    text: |
        az network private-dns record-set cname delete --name MyRecordSet --resource-group MyResourceGroup --subscription MySubscription --yes --zone-name www.mysite.com
    crafted: true
"""

helps['network private-dns record-set cname list'] = """
type: command
short-summary: List the CNAME record set in a zone.
examples:
  - name: List the CNAME record set in a zone.
    text: az network private-dns record-set cname list -g MyResourceGroup -z www.mysite.com
"""

helps['network private-dns record-set cname remove-record'] = """
type: command
short-summary: Remove a CNAME record from its record set.
long-summary: >
    By default, if the last record in a set is removed, the record set is deleted.
    To retain the empty record set, include --keep-empty-record-set.
examples:
  - name: Remove a CNAME record from its record set.
    text: |
        az network private-dns record-set cname remove-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -c www.contoso.com
"""

helps['network private-dns record-set cname set-record'] = """
type: command
short-summary: Set the value of a CNAME record.
examples:
  - name: Set the value of a CNAME record.
    text: |
        az network private-dns record-set cname set-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -c www.contoso.com
"""

helps['network private-dns record-set cname show'] = """
type: command
short-summary: Get the details of a CNAME record set.
examples:
  - name: Get the details of a CNAME record set.
    text: az network private-dns record-set cname show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
  - name: Get the details of a CNAME record set (autogenerated)
    text: |
        az network private-dns record-set cname show --name MyRecordSet --resource-group MyResourceGroup --subscription MySubscription --zone-name www.mysite.com
    crafted: true
"""

helps['network private-dns record-set cname update'] = """
type: command
short-summary: Update a CNAME record set.
examples:
  - name: Update a CNAME record set.
    text: |
        az network private-dns record-set cname update -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet --metadata owner=WebTeam
"""

helps['network private-dns record-set list'] = """
type: command
short-summary: List all record sets within a Private DNS zone.
examples:
  - name: List all "@" record sets within this zone.
    text: |
        az network private-dns record-set list -g MyResourceGroup -z www.mysite.com \\
            --query "[?name=='@']"
"""

helps['network private-dns record-set mx'] = """
type: group
short-summary: Manage Private DNS MX records.
"""

helps['network private-dns record-set mx add-record'] = """
type: command
short-summary: Add an MX record.
examples:
  - name: Add an MX record.
    text: |
        az network private-dns record-set mx add-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -e mail.mysite.com -p 10
"""

helps['network private-dns record-set mx create'] = """
type: command
short-summary: Create an empty MX record set.
examples:
  - name: Create an empty MX record set.
    text: az network private-dns record-set mx create -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set mx delete'] = """
type: command
short-summary: Delete an MX record set and all associated records.
examples:
  - name: Delete an MX record set and all associated records.
    text: az network private-dns record-set mx delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set mx list'] = """
type: command
short-summary: List all MX record sets in a zone.
examples:
  - name: List all MX record sets in a zone.
    text: az network private-dns record-set mx list -g MyResourceGroup -z www.mysite.com
"""

helps['network private-dns record-set mx remove-record'] = """
type: command
short-summary: Remove an MX record from its record set.
long-summary: >
    By default, if the last record in a set is removed, the record set is deleted.
    To retain the empty record set, include --keep-empty-record-set.
examples:
  - name: Remove an MX record from its record set.
    text: |
        az network private-dns record-set mx remove-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -e mail.mysite.com -p 10
"""

helps['network private-dns record-set mx show'] = """
type: command
short-summary: Get the details of an MX record set.
examples:
  - name: Get the details of an MX record set.
    text: az network private-dns record-set mx show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set mx update'] = """
type: command
short-summary: Update an MX record set.
examples:
  - name: Update an MX record set.
    text: |
        az network private-dns record-set mx update -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet --metadata owner=WebTeam
"""

helps['network private-dns record-set ptr'] = """
type: group
short-summary: Manage Private DNS PTR records.
"""

helps['network private-dns record-set ptr add-record'] = """
type: command
short-summary: Add a PTR record.
examples:
  - name: Add a PTR record.
    text: |
        az network private-dns record-set ptr add-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -d another.site.com
"""

helps['network private-dns record-set ptr create'] = """
type: command
short-summary: Create an empty PTR record set.
examples:
  - name: Create an empty PTR record set.
    text: az network private-dns record-set ptr create -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set ptr delete'] = """
type: command
short-summary: Delete a PTR record set and all associated records.
examples:
  - name: Delete a PTR record set and all associated records.
    text: az network private-dns record-set ptr delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set ptr list'] = """
type: command
short-summary: List all PTR record sets in a zone.
examples:
  - name: List all PTR record sets in a zone.
    text: az network private-dns record-set ptr list -g MyResourceGroup -z www.mysite.com
"""

helps['network private-dns record-set ptr remove-record'] = """
type: command
short-summary: Remove a PTR record from its record set.
long-summary: >
    By default, if the last record in a set is removed, the record set is deleted.
    To retain the empty record set, include --keep-empty-record-set.
examples:
  - name: Remove a PTR record from its record set.
    text: |
        az network private-dns record-set ptr remove-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -d another.site.com
"""

helps['network private-dns record-set ptr show'] = """
type: command
short-summary: Get the details of a PTR record set.
examples:
  - name: Get the details of a PTR record set.
    text: az network private-dns record-set ptr show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set ptr update'] = """
type: command
short-summary: Update a PTR record set.
examples:
  - name: Update a PTR record set.
    text: |
        az network private-dns record-set ptr update -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet --metadata owner=WebTeam
"""

helps['network private-dns record-set soa'] = """
type: group
short-summary: Manage Private DNS SOA record.
"""

helps['network private-dns record-set soa show'] = """
type: command
short-summary: Get the details of an SOA record.
examples:
  - name: Get the details of an SOA record.
    text: az network private-dns record-set soa show -g MyResourceGroup -z www.mysite.com
"""

helps['network private-dns record-set soa update'] = """
type: command
short-summary: Update properties of an SOA record.
examples:
  - name: Update properties of an SOA record.
    text: |
        az network private-dns record-set soa update -g MyResourceGroup -z www.mysite.com \\
            -e myhostmaster.mysite.com
"""

helps['network private-dns record-set srv'] = """
type: group
short-summary: Manage Private DNS SRV records.
"""

helps['network private-dns record-set srv add-record'] = """
type: command
short-summary: Add an SRV record.
examples:
  - name: Add an SRV record.
    text: |
        az network private-dns record-set srv add-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -t webserver.mysite.com -r 8081 -p 10 -w 10
"""

helps['network private-dns record-set srv create'] = """
type: command
short-summary: Create an empty SRV record set.
examples:
  - name: Create an empty SRV record set.
    text: |
        az network private-dns record-set srv create -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet
  - name: Create an empty SRV record set. (autogenerated)
    text: |
        az network private-dns record-set srv create --name MyRecordSet --resource-group MyResourceGroup --ttl 300 --zone-name www.mysite.com
    crafted: true
"""

helps['network private-dns record-set srv delete'] = """
type: command
short-summary: Delete an SRV record set and all associated records.
examples:
  - name: Delete an SRV record set and all associated records.
    text: az network private-dns record-set srv delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set srv list'] = """
type: command
short-summary: List all SRV record sets in a zone.
examples:
  - name: List all SRV record sets in a zone.
    text: az network private-dns record-set srv list -g MyResourceGroup -z www.mysite.com
"""

helps['network private-dns record-set srv remove-record'] = """
type: command
short-summary: Remove an SRV record from its record set.
long-summary: >
    By default, if the last record in a set is removed, the record set is deleted.
    To retain the empty record set, include --keep-empty-record-set.
examples:
  - name: Remove an SRV record from its record set.
    text: |
        az network private-dns record-set srv remove-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -t webserver.mysite.com -r 8081 -p 10 -w 10
"""

helps['network private-dns record-set srv show'] = """
type: command
short-summary: Get the details of an SRV record set.
examples:
  - name: Get the details of an SRV record set.
    text: az network private-dns record-set srv show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set srv update'] = """
type: command
short-summary: Update an SRV record set.
examples:
  - name: Update an SRV record set.
    text: |
        az network private-dns record-set srv update -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet --metadata owner=WebTeam
"""

helps['network private-dns record-set txt'] = """
type: group
short-summary: Manage Private DNS TXT records.
"""

helps['network private-dns record-set txt add-record'] = """
type: command
short-summary: Add a TXT record.
examples:
  - name: Add a TXT record.
    text: |
        az network private-dns record-set txt add-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -v Owner=WebTeam
"""

helps['network private-dns record-set txt create'] = """
type: command
short-summary: Create an empty TXT record set.
examples:
  - name: Create an empty TXT record set.
    text: az network private-dns record-set txt create -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set txt delete'] = """
type: command
short-summary: Delete a TXT record set and all associated records.
examples:
  - name: Delete a TXT record set and all associated records.
    text: az network private-dns record-set txt delete -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set txt list'] = """
type: command
short-summary: List all TXT record sets in a zone.
examples:
  - name: List all TXT record sets in a zone.
    text: az network private-dns record-set txt list -g MyResourceGroup -z www.mysite.com
"""

helps['network private-dns record-set txt remove-record'] = """
type: command
short-summary: Remove a TXT record from its record set.
long-summary: >
    By default, if the last record in a set is removed, the record set is deleted.
    To retain the empty record set, include --keep-empty-record-set.
examples:
  - name: Remove a TXT record from its record set.
    text: |
        az network private-dns record-set txt remove-record -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet -v Owner=WebTeam
"""

helps['network private-dns record-set txt show'] = """
type: command
short-summary: Get the details of a TXT record set.
examples:
  - name: Get the details of a TXT record set.
    text: az network private-dns record-set txt show -g MyResourceGroup -z www.mysite.com -n MyRecordSet
"""

helps['network private-dns record-set txt update'] = """
type: command
short-summary: Update a TXT record set.
examples:
  - name: Update a TXT record set.
    text: |
        az network private-dns record-set txt update -g MyResourceGroup -z www.mysite.com \\
            -n MyRecordSet --metadata owner=WebTeam
"""

helps['network private-dns zone'] = """
type: group
short-summary: Manage Private DNS zones.
"""

helps['network private-dns zone create'] = """
type: command
short-summary: Create a Private DNS zone.
parameters:
  - name: --tags
    short-summary: Resource tags for the Private DNS zone.
examples:
  - name: Create a Private DNS zone using a fully qualified domain name.
    text: >
        az network private-dns zone create -g MyResourceGroup -n www.mysite.com
"""

helps['network private-dns zone delete'] = """
type: command
short-summary: Delete a Private DNS zone.
long-summary: All DNS records in the zone will also be deleted. This operation cannot be undone. Private DNS zone cannot be deleted unless all virtual network links to it are removed.
examples:
  - name: Delete a Private DNS zone using a fully qualified domain name.
    text: >
        az network private-dns zone delete -g MyResourceGroup -n www.mysite.com
"""

helps['network private-dns zone list'] = """
type: command
short-summary: List Private DNS zones.
examples:
  - name: List Private DNS zones in a resource group.
    text: >
        az network private-dns zone list -g MyResourceGroup
"""

helps['network private-dns zone show'] = """
type: command
short-summary: Get a Private DNS zone.
examples:
  - name: Get a Private DNS zone using a fully qualified domain name.
    text: >
        az network private-dns zone show -g MyResourceGroup -n www.mysite.com
"""

helps['network private-dns zone update'] = """
type: command
short-summary: Update a Private DNS zone's properties. Does not modify Private DNS records or virtual network links within the zone.
parameters:
  - name: --tags
    short-summary: Resource tags for the Private DNS zone.
examples:
  - name: Update a Private DNS zone properties to change the user-defined value of a previously set tag.
    text: >
        az network private-dns zone update -g MyResourceGroup -n www.mysite.com --tags CostCenter=Marketing
"""

helps['network private-dns zone wait'] = """
type: command
short-summary: Place the CLI in a waiting state until a condition of the Private DNS zone is met.
examples:
  - name: Pause executing next line of CLI script until the Private DNS zone is successfully provisioned.
    text: az network private-dns zone wait -g MyResourceGroup -n www.mysite.com --created
"""

helps['network private-dns zone export'] = """
type: command
short-summary: Export a Private DNS zone as a DNS zone file.
examples:
  - name: Export a Private DNS zone as a DNS zone file.
    text: >
        az network private-dns zone export -g MyResourceGroup -n www.mysite.com -f mysite_com_zone.txt
"""

helps['network private-dns zone import'] = """
type: command
short-summary: Create a Private DNS zone using a DNS zone file.
examples:
  - name: Import a local zone file into a Private DNS zone resource.
    text: >
        az network private-dns zone import -g MyResourceGroup -n MyZone -f /path/to/zone/file
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import shlex
import json
from io import StringIO

from azure.cli.command_modules.resource.aaz.latest.policy.assignment._create import Create as AssignmentCreate
from azure.cli.command_modules.resource.aaz.latest.policy.assignment._delete import Delete as AssignmentDelete
from azure.cli.command_modules.resource.aaz.latest.policy.assignment._list import List as AssignmentList
from azure.cli.command_modules.resource.aaz.latest.policy.assignment._show import Show as AssignmentShow
from azure.cli.command_modules.resource.aaz.latest.policy.assignment._update import Update as AssignmentUpdate

from azure.cli.command_modules.resource.aaz.latest.policy.assignment.identity._assign \
    import Assign as AssignmentIdentityAssign
from azure.cli.command_modules.resource.aaz.latest.policy.assignment.identity._remove \
    import Remove as AssignmentIdentityRemove
from azure.cli.command_modules.resource.aaz.latest.policy.assignment.identity._show \
    import Show as AssignmentIdentityShow

from azure.cli.command_modules.resource.aaz.latest.policy.assignment.non_compliance_message._create \
    import Create as NonComplianceMessageCreate
from azure.cli.command_modules.resource.aaz.latest.policy.assignment.non_compliance_message._delete \
    import Delete as NonComplianceMessageDelete
from azure.cli.command_modules.resource.aaz.latest.policy.assignment.non_compliance_message._list \
    import List as NonComplianceMessageList
from azure.cli.command_modules.resource.aaz.latest.policy.assignment.non_compliance_message._show \
    import Show as NonComplianceMessageShow
from azure.cli.command_modules.resource.aaz.latest.policy.assignment.non_compliance_message._update \
    import Update as NonComplianceMessageUpdate

from azure.cli.command_modules.resource.aaz.latest.policy.definition._create import Create as DefinitionCreate
from azure.cli.command_modules.resource.aaz.latest.policy.definition._delete import Delete as DefinitionDelete
from azure.cli.command_modules.resource.aaz.latest.policy.definition._list import List as DefinitionList
from azure.cli.command_modules.resource.aaz.latest.policy.definition._show import Show as DefinitionShow
from azure.cli.command_modules.resource.aaz.latest.policy.definition._update import Update as DefinitionUpdate

from azure.cli.command_modules.resource.aaz.latest.policy.exemption._create import Create as ExemptionCreate
from azure.cli.command_modules.resource.aaz.latest.policy.exemption._delete import Delete as ExemptionDelete
from azure.cli.command_modules.resource.aaz.latest.policy.exemption._list import List as ExemptionList
from azure.cli.command_modules.resource.aaz.latest.policy.exemption._show import Show as ExemptionShow
from azure.cli.command_modules.resource.aaz.latest.policy.exemption._update import Update as ExemptionUpdate

from azure.cli.command_modules.resource.aaz.latest.policy.set_definition._create import Create as SetDefinitionCreate
from azure.cli.command_modules.resource.aaz.latest.policy.set_definition._delete import Delete as SetDefinitionDelete
from azure.cli.command_modules.resource.aaz.latest.policy.set_definition._list import List as SetDefinitionList
from azure.cli.command_modules.resource.aaz.latest.policy.set_definition._show import Show as SetDefinitionShow
from azure.cli.command_modules.resource.aaz.latest.policy.set_definition._update import Update as SetDefinitionUpdate

from azure.cli.command_modules.resource._client_factory import _resource_policy_client_factory
from azure.cli.core.aaz import has_value, AAZResourceGroupNameArg, AAZStrArg, AAZBoolArg
from azure.cli.core.azclierror import InvalidArgumentValueError, ArgumentUsageError
from azure.cli.core.commands.client_factory import get_subscription_id
from azure.cli.core.decorators import Completer
from azure.core.exceptions import ResourceNotFoundError
from azure.mgmt.core.tools import is_valid_resource_id, parse_resource_id

# --------------------------------------------------------------------------------------------
# Customization for auto-generated Azure Policy CLI commands (cstack: 5/9/2025)
#
# Notes on auto-generation
#  - The behavior of the auto-generated command code required significant customization to
#    work around differences from previous custom code. All customization is performed by
#    overriding generated code. No modification of generated code is present.
#  - The Common class is used for shard methods needed by the customization code.
#  - The Completers class hosts command line completers that were previously registered in
#    the custom commands.
#  - The other classes in this file override generated classes to evoke the correct behavior
#    matching the fully custom commands previously shipped.
# --------------------------------------------------------------------------------------------


# Shared code for policy command customization
class Common:

    @staticmethod
    # Ensure that both --policy and -policy-set-definition are not specified, or that one of them is specified
    def ValidatePolicyDefinitionId(ctx):
        if bool(has_value(ctx.args.policy)) == bool(has_value(ctx.args.policy_set_definition)):
            raise ArgumentUsageError('usage error: --policy NAME_OR_ID | --policy-set-definition NAME_OR_ID')

    # Ensure that --scope is not used with other scoping parameters
    @staticmethod
    def ValidateScope(ctx):
        if has_value(ctx.args.scope) and has_value(ctx.args.resource_group):
            raise ArgumentUsageError('usage error: --scope SCOPE | --resource-group NAME')
        # Should also validate that both --scope and --subscription are not specified together. Unfortunately
        # it's not possible to make this check since there is no way to determine whether the user entered
        # --subscription on the command line. Current behavior is to ignore --subscription if both are provided.

    # If --scope is not provided, set it by subscription or resource group scope
    @staticmethod
    def PopulateScopeFromContext(ctx, cli_ctx):
        if not has_value(ctx.args.scope):
            subscription_id = get_subscription_id(cli_ctx)
            if has_value(ctx.args.resource_group):
                ctx.args.scope = f"/subscriptions/{subscription_id}/resourceGroups/{ctx.args.resource_group}"
            else:
                ctx.args.scope = f"/subscriptions/{subscription_id}"

    # Create role assignment for the system assigned identity of the policy assignment
    # pylint: disable=protected-access
    @staticmethod
    def CreateRoleAssignment(ctx, cli_ctx, assignment):
        from azure.cli.core.commands.arm import assign_identity
        if has_value(ctx.args.identity_scope):
            identity_role = None
            if has_value(ctx.args.role):
                identity_role = ctx.args.role
            assign_identity(
                cli_ctx, lambda: assignment, lambda resource: assignment,
                identity_role._data, ctx.args.identity_scope._data)

    # Implement default identity type behavior for policy assignment create
    @staticmethod
    def ResolveCreateIdentityType(ctx):
        if has_value(ctx.args.identity_scope) or has_value(ctx.args.location) or has_value(ctx.args.role):
            Common.ResolveIdentityType(ctx)

    # Set identity type to system assigned if not specified
    @staticmethod
    def ResolveIdentityType(ctx):
        if not has_value(ctx.args.mi_system_assigned) and not has_value(ctx.args.mi_user_assigned):
            ctx.args.mi_system_assigned = 'True'

    # Implement default scope behavior for policy assignment and exemption list commands
    # pylint: disable=protected-access
    # pylint: disable=attribute-defined-outside-init
    def ResolveScopeForList(self):
        ctx = self.ctx
        if has_value(ctx.args.scope):
            scope_parts = ctx.args.scope._data.split('/')
            if scope_parts[1] == 'providers' and len(scope_parts) > 4:
                ctx.args.management_group = scope_parts[4]
                ctx.args.resource_group = None
            elif scope_parts[1] == 'subscriptions':
                # store subscription from scope for later use
                self.subscription_from_scope = scope_parts[2]
                if len(scope_parts) > 3:
                    ctx.args.resource_group = scope_parts[4]
                else:
                    ctx.args.resource_group = None
            else:
                raise InvalidArgumentValueError("Invalid value in --scope: '%s'" % ctx.args.scope._data)

    # Implement default name behavior for policy assignment and exemption create commands
    @staticmethod
    def GenerateNameIfNone(ctx):
        if not has_value(ctx.args.name):
            import base64
            import uuid
            ctx.args.name = (base64.urlsafe_b64encode(uuid.uuid4().bytes).decode())[:-2]

    # Get policy definition ID from policy name if not specified
    # pylint: disable=protected-access
    @staticmethod
    def ResolvePolicyId(ctx):
        policy_id = ctx.args.policy._data or ctx.args.policy_set_definition._data
        if not is_valid_resource_id(policy_id):
            definition = None
            name = policy_id.split('/')[-1]
            command_string = None
            if has_value(ctx.args.policy):
                command_string = f"policy definition show --name {name}"
            else:
                command_string = f"policy set-definition show --name {name}"

            if policy_id.startswith('/providers'):
                management_group_name = policy_id.split('/')[4]
                command_string = command_string + f" --management-group {management_group_name}"

            try:
                definition = Common.run_cli_deserialize(command_string)
            except Exception as ex:
                raise InvalidArgumentValueError(
                    f"Invalid value in --policy or --policy-set-definition: '{policy_id}'") from ex

            policy_id = definition.get('id')

        ctx.args.policy_set_definition = policy_id

    # Get user assigned identity IDs from names if not specified
    # pylint: disable=protected-access
    # pylint: disable=consider-using-enumerate
    @staticmethod
    def ResolveUserAssignedIdentityId(ctx, cli_ctx):
        from azure.cli.command_modules.resource.custom import _get_resource_id
        if has_value(ctx.args.mi_user_assigned):
            resource_group = ctx.args.resource_group._data if has_value(ctx.args.resource_group) else None
            user_assigned_identities = ctx.args.mi_user_assigned
            for i in range(len(user_assigned_identities)):
                user_assigned_identity = user_assigned_identities[i]._data
                if not is_valid_resource_id(user_assigned_identity):
                    user_assigned_identities[i] = _get_resource_id(
                        cli_ctx, user_assigned_identity, resource_group,
                        'userAssignedIdentities', 'Microsoft.ManagedIdentity')

    # Helper function to check whether a scope is a management group scope
    @staticmethod
    def _is_management_group_scope(scope):
        return scope is not None and scope.lower().startswith("/providers/microsoft.management/managementgroups")

    # Validate that not_scopes are valid subscription or management group scopes
    # pylint: disable=protected-access
    @staticmethod
    def ValidateNotScopes(ctx):
        not_scopes = ctx.args.not_scopes
        if has_value(not_scopes):
            for not_scope in not_scopes:
                id_arg = not_scope._data
                id_parts = parse_resource_id(id_arg)
                if not (id_parts.get('subscription') or Common._is_management_group_scope(id_arg)):
                    raise InvalidArgumentValueError("Invalid resource ID value in --not-scopes: '%s'" % id_arg)

    # Fix not_scopes output to be a list of strings instead of a single string
    def AdjustNotScopesOutput(self):
        notScopes = self.deserialize_output(self.ctx.vars.instance.properties.notScopes)
        if notScopes and len(notScopes) == 1:
            split = notScopes[0].split(' ')
            self.ctx.vars.instance.properties.notScopes = split

    # Change UTC format from Zulu to +00:00
    @staticmethod
    def AdjustUTCFormat(exemption):
        if exemption.properties.expires_on:
            exemption.properties.expires_on = exemption.properties.expires_on._data.replace('Z', '+00:00')

    # Apply filter to list commands based on --disable-scope-strict-match argument
    @staticmethod
    def ApplyListFilter(ctx):
        if not has_value(ctx.args.filter):
            if has_value(ctx.args.disable_scope_strict_match):
                ctx.args.filter = "atScopeAndBelow()"
            else:
                ctx.args.filter = "atExactScope()"

    # Helper to execute Azure CLI command and return the output as JSON
    @staticmethod
    def run_cli_deserialize(command):
        from azure.cli.core.util import run_az_cmd
        stdout_buf = StringIO()
        result = run_az_cmd(shlex.split(command), out_file=stdout_buf)
        exit_code = result.exit_code or 0
        output = stdout_buf.getvalue()

        # Deserialize if the command was successful
        if exit_code == 0:
            return json.loads(output)

        return None


# Completers for policy command arguments
class Completers:

    @staticmethod
    @Completer
    def get_policy_definition_completion_list(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
        policy_client = _resource_policy_client_factory(cmd.cli_ctx)
        result = policy_client.policy_definitions.list(filter="policyType eq 'Custom'")
        return [i.name for i in result]

    @staticmethod
    @Completer
    def get_policy_set_completion_list(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
        policy_client = _resource_policy_client_factory(cmd.cli_ctx)
        result = policy_client.policy_set_definitions.list(filter="policyType eq 'Custom'")
        return [i.name for i in result]

    @staticmethod
    @Completer
    def get_policy_assignment_completion_list(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
        policy_client = _resource_policy_client_factory(cmd.cli_ctx)
        result = policy_client.policy_assignments.list()
        return [i.name for i in result]

    @staticmethod
    @Completer
    def get_policy_exemption_completion_list(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
        policy_client = _resource_policy_client_factory(cmd.cli_ctx)
        result = policy_client.policy_exemptions.list()
        return [i.name for i in result]


class PolicyAssignmentCreate(AssignmentCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._required = False                # pylint: disable=protected-access
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        args_schema.policy = AAZStrArg(
            options=['--policy'],
            help='The name or resource ID of the policy definition or policy set definition to be assigned.')
        args_schema.identity_scope = AAZStrArg(
            options=['--identity-scope'],
            help='Scope that the system assigned identity can access.')
        args_schema.role = AAZStrArg(
            options=['--role'],
            help='Role name or id that will be assigned to the managed identity.')
        return args_schema

    def pre_operations(self):
        Common.ValidatePolicyDefinitionId(self.ctx)
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)
        Common.ValidateNotScopes(self.ctx)
        Common.GenerateNameIfNone(self.ctx)
        Common.ResolvePolicyId(self.ctx)
        Common.ResolveUserAssignedIdentityId(self.ctx, self.cli_ctx)
        Common.ResolveCreateIdentityType(self.ctx)

    def post_operations(self):
        Common.CreateRoleAssignment(self.ctx, self.cli_ctx, self.ctx.vars.instance)

    # pylint: disable=arguments-differ
    def _output(self):
        Common.AdjustNotScopesOutput(self)
        result = super()._output(self)
        return result


class PolicyAssignmentDelete(AssignmentDelete):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_assignment_completion_list
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)


class PolicyAssignmentList(AssignmentList):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope = AAZStrArg(
            options=['--scope'],
            help='Scope at which to list applicable policy assignments. '
                 'If scope is not provided, the scope will be the implied or specified subscription.')
        args_schema.disable_scope_strict_match = AAZBoolArg(
            options=['-d', '--disable-scope-strict-match'],
            help='Include policy assignments either inherited from parent scopes or at child scopes.')
        return args_schema

    # subscription provided by --scope argument (may be different from context subscription)
    def __init__(self, loader):
        super().__init__(loader)
        self.subscription_from_scope = None

    class PolicyAssignmentsListForManagementGroup(AssignmentList.PolicyAssignmentsListForManagementGroup):
        pass

    class PolicyAssignmentsListForResourceGroup(AssignmentList.PolicyAssignmentsListForResourceGroup):
        pass

    class PolicyAssignmentsList(AssignmentList.PolicyAssignmentsList):
        def __init__(self, ctx, subscription_from_scope):
            super().__init__(ctx)
            self.subscription_from_scope = subscription_from_scope

        @property
        def url(self):
            if self.subscription_from_scope is not None:
                return self.client.format_url(
                    "/subscriptions/{subscriptionId}/providers/Microsoft.Authorization/policyAssignments",
                    subscriptionId=self.subscription_from_scope
                )

            return super().url

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.ResolveScopeForList(self)
        Common.ApplyListFilter(self.ctx)

    def _execute_operations(self):
        self.pre_operations()

        if has_value(self.ctx.args.management_group):
            self.PolicyAssignmentsListForManagementGroup(ctx=self.ctx)()
        elif has_value(self.ctx.args.resource_group):
            self.PolicyAssignmentsListForResourceGroup(ctx=self.ctx)()
        else:
            self.PolicyAssignmentsList(ctx=self.ctx, subscription_from_scope=self.subscription_from_scope)()

        self.post_operations()


class PolicyAssignmentShow(AssignmentShow):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_assignment_completion_list
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)


class PolicyAssignmentUpdate(AssignmentUpdate):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_assignment_completion_list
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        args_schema.policy = AAZStrArg(
            options=['--policy'],
            help='The name or resource ID of the policy definition or policy set definition to be assigned.')
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)
        Common.ValidateNotScopes(self.ctx)

    # pylint: disable=arguments-differ
    def _output(self):
        Common.AdjustNotScopesOutput(self)
        result = super()._output(self)
        return result


class PolicyAssignmentIdentityAssign(AssignmentIdentityAssign):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        args_schema.identity_scope = AAZStrArg(
            options=['--identity-scope'],
            help='Scope that the system assigned identity can access.')
        args_schema.role = AAZStrArg(
            options=['--role'],
            help='Role name or id that will be assigned to the managed identity.')
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)
        Common.ResolveIdentityType(self.ctx)

    def post_operations(self):
        Common.CreateRoleAssignment(self.ctx, self.cli_ctx, self.ctx.vars.instance)


class PolicyAssignmentIdentityRemove(AssignmentIdentityRemove):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)
        Common.ResolveIdentityType(self.ctx)


class PolicyAssignmentIdentityShow(AssignmentIdentityShow):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)


class PolicyAssignmentNonComplianceMessageCreate(NonComplianceMessageCreate):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._required = False
        args_schema.policy_definition_reference_id._required = False
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)


class PolicyAssignmentNonComplianceMessageDelete(NonComplianceMessageDelete):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._required = False
        args_schema.policy_definition_reference_id._required = False
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)


class PolicyAssignmentNonComplianceMessageList(NonComplianceMessageList):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)

    # pylint: disable=arguments-differ
    def _output(self):
        value = self.ctx.selectors.subresource.get()
        if has_value(value):
            result = self.deserialize_output(value, client_flatten=True)
        else:
            result = None
        return result


class PolicyAssignmentNonComplianceMessageShow(NonComplianceMessageShow):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._required = False
        args_schema.policy_definition_reference_id._required = False
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)


class PolicyAssignmentNonComplianceMessageUpdate(NonComplianceMessageUpdate):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.policy_definition_reference_id._required = False
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)


class PolicyDefinitionCreate(DefinitionCreate):

    class PolicyDefinitionsCreateOrUpdateAtManagementGroup(
            DefinitionCreate.PolicyDefinitionsCreateOrUpdateAtManagementGroup):
        pass

    class PolicyDefinitionsCreateOrUpdate(DefinitionCreate.PolicyDefinitionsCreateOrUpdate):
        pass

    def pre_operations(self):
        pass

    def _execute_operations(self):
        self.pre_operations()

        if has_value(self.ctx.args.name) and has_value(self.ctx.args.management_group):
            self.PolicyDefinitionsCreateOrUpdateAtManagementGroup(ctx=self.ctx)()
        else:
            self.PolicyDefinitionsCreateOrUpdate(ctx=self.ctx)()

        self.post_operations()

    def post_operations(self):
        pass


class PolicyDefinitionDelete(DefinitionDelete):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_definition_completion_list
        return args_schema


class PolicyDefinitionList(DefinitionList):

    class PolicyDefinitionsListByManagementGroup(DefinitionList.PolicyDefinitionsListByManagementGroup):
        pass

    class PolicyDefinitionsList(DefinitionList.PolicyDefinitionsList):
        pass

    def pre_operations(self):
        pass

    def _execute_operations(self):
        self.pre_operations()

        if has_value(self.ctx.args.management_group):
            self.PolicyDefinitionsListByManagementGroup(ctx=self.ctx)()
        else:
            self.PolicyDefinitionsList(ctx=self.ctx)()

        self.post_operations()

    def post_operations(self):
        pass


class PolicyDefinitionShow(DefinitionShow):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_definition_completion_list
        return args_schema

    class PolicyDefinitionsGetBuiltIn(DefinitionShow.PolicyDefinitionsGet):

        @property
        def url(self):
            return self.client.format_url(
                "/providers/Microsoft.Authorization/policyDefinitions/{policyDefinitionName}",
                **self.url_parameters
            )

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    'policyDefinitionName', self.ctx.args.name,
                    required=True,
                )
            }
            return parameters

    class PolicyDefinitionsGetAtManagementGroup(DefinitionShow.PolicyDefinitionsGetAtManagementGroup):
        pass

    class PolicyDefinitionsGet(DefinitionShow.PolicyDefinitionsGet):
        pass

    def pre_operations(self):
        pass

    def _execute_operations(self):

        self.pre_operations()
        done = False
        if has_value(self.ctx.args.name) and not has_value(self.ctx.args.management_group):
            try:
                self.PolicyDefinitionsGetBuiltIn(ctx=self.ctx)()
                done = True
            except ResourceNotFoundError:
                pass

        if not done and has_value(self.ctx.args.name) and has_value(self.ctx.args.management_group):
            self.PolicyDefinitionsGetAtManagementGroup(ctx=self.ctx)()
        elif not done:
            self.PolicyDefinitionsGet(ctx=self.ctx)()

        self.post_operations()

    def post_operations(self):
        pass


class PolicyDefinitionUpdate(DefinitionUpdate):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_definition_completion_list
        return args_schema

    class PolicyDefinitionsCreateOrUpdateAtManagementGroup(
            DefinitionUpdate.PolicyDefinitionsCreateOrUpdateAtManagementGroup):
        pass

    class PolicyDefinitionsCreateOrUpdate(DefinitionUpdate.PolicyDefinitionsCreateOrUpdate):
        pass

    class PolicyDefinitionsGetAtManagementGroup(DefinitionUpdate.PolicyDefinitionsGetAtManagementGroup):
        pass

    class PolicyDefinitionsGet(DefinitionUpdate.PolicyDefinitionsGet):
        pass

    # pylint: disable=too-few-public-methods
    class InstanceUpdateByJson(DefinitionUpdate.InstanceUpdateByJson):
        pass

    # pylint: disable=too-few-public-methods
    class InstanceUpdateByGeneric(DefinitionUpdate.InstanceUpdateByGeneric):
        pass

    def pre_operations(self):
        pass

    def _execute_operations(self):
        self.pre_operations()

        if has_value(self.ctx.args.name) and has_value(self.ctx.args.management_group):
            self.PolicyDefinitionsGetAtManagementGroup(ctx=self.ctx)()
            self.InstanceUpdateByJson(ctx=self.ctx)()
            self.InstanceUpdateByGeneric(ctx=self.ctx)()
            self.PolicyDefinitionsCreateOrUpdateAtManagementGroup(ctx=self.ctx)()
        else:
            self.PolicyDefinitionsGet(ctx=self.ctx)()
            self.InstanceUpdateByJson(ctx=self.ctx)()
            self.InstanceUpdateByGeneric(ctx=self.ctx)()
            self.PolicyDefinitionsCreateOrUpdate(ctx=self.ctx)()

        self.post_operations()

    def post_operations(self):
        pass


class PolicyExemptionCreate(ExemptionCreate):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._required = False            # pylint: disable=protected-access
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)
        Common.GenerateNameIfNone(self.ctx)

    def post_operations(self):
        Common.AdjustUTCFormat(self.ctx.vars.instance)


class PolicyExemptionDelete(ExemptionDelete):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_exemption_completion_list
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)


class PolicyExemptionList(ExemptionList):

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.scope = AAZStrArg(
            options=['--scope'],
            help='Scope at which to list applicable policy exemptions. '
                 'If scope is not provided, the scope will be the implied or specified subscription.')
        args_schema.disable_scope_strict_match = AAZBoolArg(
            options=['-d', '--disable-scope-strict-match'],
            help='Include policy exemptions either inherited from parent scopes or at child scopes.')
        return args_schema

    # subscription provided by --scope argument (may be different from context subscription)
    def __init__(self, loader):
        super().__init__(loader)
        self.subscription_from_scope = None

    class PolicyExemptionsListForManagementGroup(ExemptionList.PolicyExemptionsListForManagementGroup):
        pass

    class PolicyExemptionsListForResourceGroup(ExemptionList.PolicyExemptionsListForResourceGroup):
        pass

    class PolicyExemptionsList(ExemptionList.PolicyExemptionsList):
        def __init__(self, ctx, subscription_from_scope):
            super().__init__(ctx)
            self.subscription_from_scope = subscription_from_scope

        @property
        def url(self):
            if self.subscription_from_scope is not None:
                return self.client.format_url(
                    "/subscriptions/{subscriptionId}/providers/Microsoft.Authorization/policyExemptions",
                    subscriptionId=self.subscription_from_scope
                )

            return super().url

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.ResolveScopeForList(self)
        Common.ApplyListFilter(self.ctx)

    def _execute_operations(self):
        self.pre_operations()

        if has_value(self.ctx.args.management_group):
            self.PolicyExemptionsListForManagementGroup(ctx=self.ctx)()
        elif has_value(self.ctx.args.resource_group):
            self.PolicyExemptionsListForResourceGroup(ctx=self.ctx)()
        else:
            self.PolicyExemptionsList(ctx=self.ctx, subscription_from_scope=self.subscription_from_scope)()

        self.post_operations()

    def post_operations(self):
        listing = self.ctx.vars.instance.value
        for item in listing:
            Common.AdjustUTCFormat(item)


class PolicyExemptionShow(ExemptionShow):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_exemption_completion_list
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)

    def post_operations(self):
        Common.AdjustUTCFormat(self.ctx.vars.instance)


class PolicyExemptionUpdate(ExemptionUpdate):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_exemption_completion_list
        args_schema.scope._required = False               # pylint: disable=protected-access
        args_schema.resource_group = AAZResourceGroupNameArg()
        return args_schema

    def pre_operations(self):
        Common.ValidateScope(self.ctx)
        Common.PopulateScopeFromContext(self.ctx, self.cli_ctx)

    def post_operations(self):
        Common.AdjustUTCFormat(self.ctx.vars.instance)


class PolicySetDefinitionCreate(SetDefinitionCreate):

    class PolicySetDefinitionsCreateOrUpdateAtManagementGroup(
            SetDefinitionCreate.PolicySetDefinitionsCreateOrUpdateAtManagementGroup):
        pass

    class PolicySetDefinitionsCreateOrUpdate(SetDefinitionCreate.PolicySetDefinitionsCreateOrUpdate):
        pass

    def pre_operations(self):
        pass

    def _execute_operations(self):
        self.pre_operations()

        if has_value(self.ctx.args.name) and has_value(self.ctx.args.management_group):
            self.PolicySetDefinitionsCreateOrUpdateAtManagementGroup(ctx=self.ctx)()
        else:
            self.PolicySetDefinitionsCreateOrUpdate(ctx=self.ctx)()

        self.post_operations()

    def post_operations(self):
        pass


class PolicySetDefinitionDelete(SetDefinitionDelete):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_set_completion_list
        return args_schema


class PolicySetDefinitionList(SetDefinitionList):

    class PolicySetDefinitionsListByManagementGroup(SetDefinitionList.PolicySetDefinitionsListByManagementGroup):
        pass

    class PolicySetDefinitionsList(SetDefinitionList.PolicySetDefinitionsList):
        pass

    def pre_operations(self):
        pass

    def _execute_operations(self):
        self.pre_operations()

        if has_value(self.ctx.args.management_group):
            self.PolicySetDefinitionsListByManagementGroup(ctx=self.ctx)()
        else:
            self.PolicySetDefinitionsList(ctx=self.ctx)()

        self.post_operations()

    def post_operations(self):
        pass


class PolicySetDefinitionShow(SetDefinitionShow):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_set_completion_list
        return args_schema

    class PolicySetDefinitionsGetBuiltIn(SetDefinitionShow.PolicySetDefinitionsGet):

        @property
        def url(self):
            return self.client.format_url(
                "/providers/Microsoft.Authorization/policySetDefinitions/{policySetDefinitionName}",
                **self.url_parameters
            )

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    'policySetDefinitionName', self.ctx.args.name,
                    required=True,
                )
            }
            return parameters

    class PolicySetDefinitionsGetAtManagementGroup(SetDefinitionShow.PolicySetDefinitionsGetAtManagementGroup):
        pass

    class PolicySetDefinitionsGet(SetDefinitionShow.PolicySetDefinitionsGet):
        pass

    def pre_operations(self):
        pass

    def _execute_operations(self):
        self.pre_operations()
        done = False
        if has_value(self.ctx.args.name) and not has_value(self.ctx.args.management_group):
            try:
                self.PolicySetDefinitionsGetBuiltIn(ctx=self.ctx)()
                done = True
            except ResourceNotFoundError:
                pass

        if not done and has_value(self.ctx.args.name) and has_value(self.ctx.args.management_group):
            self.PolicySetDefinitionsGetAtManagementGroup(ctx=self.ctx)()
        elif not done:
            self.PolicySetDefinitionsGet(ctx=self.ctx)()

        self.post_operations()

    def post_operations(self):
        pass


class PolicySetDefinitionUpdate(SetDefinitionUpdate):

    # pylint: disable=protected-access
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._completer = Completers.get_policy_set_completion_list
        return args_schema

    class PolicySetDefinitionsCreateOrUpdateAtManagementGroup(
            SetDefinitionUpdate.PolicySetDefinitionsCreateOrUpdateAtManagementGroup):
        pass

    class PolicySetDefinitionsCreateOrUpdate(SetDefinitionUpdate.PolicySetDefinitionsCreateOrUpdate):
        pass

    class PolicySetDefinitionsGetAtManagementGroup(SetDefinitionUpdate.PolicySetDefinitionsGetAtManagementGroup):
        pass

    class PolicySetDefinitionsGet(SetDefinitionUpdate.PolicySetDefinitionsGet):
        pass

    # pylint: disable=too-few-public-methods
    class InstanceUpdateByJson(SetDefinitionUpdate.InstanceUpdateByJson):
        pass

    # pylint: disable=too-few-public-methods
    class InstanceUpdateByGeneric(SetDefinitionUpdate.InstanceUpdateByGeneric):
        pass

    def pre_operations(self):
        pass

    def _execute_operations(self):
        self.pre_operations()

        if has_value(self.ctx.args.name) and has_value(self.ctx.args.management_group):
            self.PolicySetDefinitionsGetAtManagementGroup(ctx=self.ctx)()
            self.InstanceUpdateByJson(ctx=self.ctx)()
            self.InstanceUpdateByGeneric(ctx=self.ctx)()
            self.PolicySetDefinitionsCreateOrUpdateAtManagementGroup(ctx=self.ctx)()
        else:
            self.PolicySetDefinitionsGet(ctx=self.ctx)()
            self.InstanceUpdateByJson(ctx=self.ctx)()
            self.InstanceUpdateByGeneric(ctx=self.ctx)()
            self.PolicySetDefinitionsCreateOrUpdate(ctx=self.ctx)()

        self.post_operations()

    def post_operations(self):
        pass



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "policy assignment create",
)
class Create(AAZCommand):
    """Create a policy assignment.

    Create a policy assignment with the given scope and name. Policy assignments apply to all resources contained within their scope. For example, when you assign a policy at resource group scope, that policy applies to all resources in the group.

    :example: Create a resource policy assignment at scope
        az policy assignment create --scope "/providers/Microsoft.Management/managementGroups/{managementGroupName}" --policy {policyName} -p "{ 'allowedLocations': { 'value': [ 'australiaeast', 'eastus', 'japaneast' ] } }"

    :example: Create a resource policy assignment and provide rule parameter values
        az Create a resource policy assignment and provide rule parameter values policy assignment create --policy {policyName} -p "{ 'allowedLocations': { 'value': [ 'australiaeast', 'eastus', 'japaneast' ] } }"

    :example: Create a resource policy assignment with a system assigned identity
        az policy assignment create --name myPolicy --policy {policyName} --mi-system-assigned --location eastus

    :example: Create a resource policy assignment with a system assigned identity with Contributor role access to the subscription
        az policy assignment create --name myPolicy --policy {policyName} --mi-system-assigned --identity-scope /subscriptions/{subscriptionId} --role Contributor --location eastus

    :example: Create a resource policy assignment with a user assigned identity
        az policy assignment create --name myPolicy --policy {policyName} -g MyResourceGroup --mi-user-assigned myAssignedId --location westus

    :example: Create a resource policy assignment with an enforcement mode
        az policy assignment create --name myPolicy --policy {policyName} --enforcement-mode DoNotEnforce
    """

    _aaz_info = {
        "version": "2024-05-01",
        "resources": [
            ["mgmt-plane", "/{scope}/providers/microsoft.authorization/policyassignments/{}", "2024-05-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help={"short-summary": "The name of the policy assignment.", "long-summary": "The name of the policy assignment is the name segment of its resource ID."},
            required=True,
            fmt=AAZStrArgFormat(
                pattern="^[^<>*%&:\\?.+/]*[^<>*%&:\\?.+/ ]+$",
            ),
        )
        _args_schema.scope = AAZStrArg(
            options=["--scope"],
            help={"short-summary": "The scope of the policy assignment.", "long-summary": "Valid scopes are: management group (format: '/providers/Microsoft.Management/managementGroups/{managementGroup}'), subscription (format: '/subscriptions/{subscriptionId}'), resource group (format: '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}', or resource (format: '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/[{parentResourcePath}/]{resourceType}/{resourceName}'. The scope of an assignment is always the part of its ID preceding '/providers/Microsoft.Authorization/policyAssignments/{policyAssignmentName}'. If scope is not provided, the scope will be the implied or specified subscription."},
            required=True,
        )

        # define Arg Group "Identity"

        _args_schema = cls._args_schema
        _args_schema.mi_system_assigned = AAZStrArg(
            options=["--assign-identity", "--system-assigned", "--mi-system-assigned"],
            arg_group="Identity",
            help={"short-summary": "Sets the system assigned managed identity.", "long-summary": "Sets the system assigned managed identity of the policy assignment."},
            blank="True",
        )
        _args_schema.mi_user_assigned = AAZListArg(
            options=["--user-assigned", "--mi-user-assigned"],
            arg_group="Identity",
            help={"short-summary": "Sets the user assigned managed identity.", "long-summary": "Sets the user assigned managed identity of the policy assignment."},
            blank=[],
        )

        mi_user_assigned = cls._args_schema.mi_user_assigned
        mi_user_assigned.Element = AAZStrArg()

        # define Arg Group "Parameters"

        _args_schema = cls._args_schema
        _args_schema.location = AAZResourceLocationArg(
            arg_group="Parameters",
            help={"short-summary": "The location of the policy assignment.", "long-summary": "The location of the policy assignment is only required when utilizing managed identity."},
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.definition_version = AAZStrArg(
            options=["--definition-version"],
            arg_group="Properties",
            help={"short-summary": "The policy version to assign.", "long-summary": "The version of the policy definition or policy set definition to assign."},
        )
        _args_schema.description = AAZStrArg(
            options=["--description"],
            arg_group="Properties",
            help={"short-summary": "Policy assignment description.", "long-summary": "Full description of the policy assignment."},
        )
        _args_schema.display_name = AAZStrArg(
            options=["--display-name"],
            arg_group="Properties",
            help={"short-summary": "The display name of the policy assignment.", "long-summary": "The display name of the policy assignment is not part of its ID, allowing for longer and more flexible naming."},
        )
        _args_schema.enforcement_mode = AAZStrArg(
            options=["-e", "--enforcement-mode"],
            arg_group="Properties",
            help={"short-summary": "The policy assignment enforcement mode.", "long-summary": "The policy assignment enforcement mode. Possible values are Default and DoNotEnforce."},
            default="Default",
            enum={"Default": "Default", "DoNotEnforce": "DoNotEnforce"},
        )
        _args_schema.metadata = AAZDictArg(
            options=["--metadata"],
            arg_group="Properties",
            help={"short-summary": "The policy assignment metadata.", "long-summary": "The policy assignment metadata. Metadata is an open-ended object and is typically a collection of key value pairs."},
        )
        _args_schema.not_scopes = AAZListArg(
            options=["--not-scopes"],
            arg_group="Properties",
            help={"short-summary": "The policy assignment excluded scopes.", "long-summary": "The collection of scopes within the policy assignment scope that it does not apply to."},
        )
        _args_schema.overrides = AAZListArg(
            options=["--overrides"],
            arg_group="Properties",
            help={"short-summary": "The policy override.", "long-summary": "Array of objects that specify resource selector values to use for this assignment."},
        )
        _args_schema.params = AAZDictArg(
            options=["-p", "--params"],
            arg_group="Properties",
            help={"short-summary": "The parameter values for the assigned policy rule.", "long-summary": "Object or file path containing the policy rule parameter values to be used when assigning the policy definition. The object keys are the parameter names."},
        )
        _args_schema.policy_set_definition = AAZStrArg(
            options=["-d", "--policy-set-definition"],
            arg_group="Properties",
            help={"short-summary": "The policy definition or policy set definition to assign.", "long-summary": "The resource ID of the policy definition (format: /{scope}/providers/Microsoft.Authorization/policyDefinitions/{name}) or policy set definition (format: /{scope}/providers/Microsoft.Authorization/policySetDefinitions/{name}) to be assigned."},
        )
        _args_schema.resource_selectors = AAZListArg(
            options=["--resource-selectors"],
            arg_group="Properties",
            help={"short-summary": "The resource selectors list to filter policies by resource properties.", "long-summary": "The collection of resource selector expressions used to filter policy assignment applicability by certain resource property values."},
        )

        metadata = cls._args_schema.metadata
        metadata.Element = AAZAnyTypeArg()

        not_scopes = cls._args_schema.not_scopes
        not_scopes.Element = AAZStrArg()

        overrides = cls._args_schema.overrides
        overrides.Element = AAZObjectArg()

        _element = cls._args_schema.overrides.Element
        _element.kind = AAZStrArg(
            options=["kind"],
            help="The override kind.",
            enum={"definitionVersion": "definitionVersion", "policyEffect": "policyEffect"},
        )
        _element.selectors = AAZListArg(
            options=["selectors"],
            help="The list of the selector expressions.",
        )
        _element.value = AAZStrArg(
            options=["value"],
            help="The value to override the policy property.",
        )

        selectors = cls._args_schema.overrides.Element.selectors
        selectors.Element = AAZObjectArg()
        cls._build_args_selector_create(selectors.Element)

        params = cls._args_schema.params
        params.Element = AAZObjectArg()

        _element = cls._args_schema.params.Element
        _element.value = AAZAnyTypeArg(
            options=["value"],
            help="The value of the parameter.",
        )

        resource_selectors = cls._args_schema.resource_selectors
        resource_selectors.Element = AAZObjectArg()

        _element = cls._args_schema.resource_selectors.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="The name of the resource selector.",
        )
        _element.selectors = AAZListArg(
            options=["selectors"],
            help="The list of the selector expressions.",
        )

        selectors = cls._args_schema.resource_selectors.Element.selectors
        selectors.Element = AAZObjectArg()
        cls._build_args_selector_create(selectors.Element)

        # define Arg Group "non-compliance-message"

        _args_schema = cls._args_schema
        _args_schema.non_compliance_messages = AAZListArg(
            options=["-m", "--non-compliance-messages"],
            arg_group="non-compliance-message",
            help="The messages that describe why a resource is non-compliant with the policy.",
        )

        non_compliance_messages = cls._args_schema.non_compliance_messages
        non_compliance_messages.Element = AAZObjectArg()

        _element = cls._args_schema.non_compliance_messages.Element
        _element.message = AAZStrArg(
            options=["m", "message"],
            help={"short-summary": "A custom non-compliance message.", "long-summary": "A message that describes the reasons a resource was determined to be non-compliant. This will be shown in 'deny' error messages and resource non-compliance results."},
            required=True,
        )
        _element.policy_definition_reference_id = AAZStrArg(
            options=["r", "policy-definition-reference-id"],
            help={"short-summary": "The policy definition reference ID.", "long-summary": "The policy definition reference ID within a policy set definition the message is intended for. This is only applicable if the policy assignment assigns a policy set definition. If not provided, the message applies to all policy definitions assigned by this policy assignment."},
        )
        return cls._args_schema

    _args_selector_create = None

    @classmethod
    def _build_args_selector_create(cls, _schema):
        if cls._args_selector_create is not None:
            _schema.in_ = cls._args_selector_create.in_
            _schema.kind = cls._args_selector_create.kind
            _schema.not_in = cls._args_selector_create.not_in
            return

        cls._args_selector_create = AAZObjectArg()

        selector_create = cls._args_selector_create
        selector_create.in_ = AAZListArg(
            options=["in"],
            help="The list of values to filter in.",
        )
        selector_create.kind = AAZStrArg(
            options=["kind"],
            help="The selector kind.",
            enum={"policyDefinitionReferenceId": "policyDefinitionReferenceId", "resourceLocation": "resourceLocation", "resourceType": "resourceType", "resourceWithoutLocation": "resourceWithoutLocation"},
        )
        selector_create.not_in = AAZListArg(
            options=["not-in"],
            help="The list of values to filter out.",
        )

        in_ = cls._args_selector_create.in_
        in_.Element = AAZStrArg()

        not_in = cls._args_selector_create.not_in
        not_in.Element = AAZStrArg()

        _schema.in_ = cls._args_selector_create.in_
        _schema.kind = cls._args_selector_create.kind
        _schema.not_in = cls._args_selector_create.not_in

    def _execute_operations(self):
        self.pre_operations()
        self.PolicyAssignmentsCreate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class PolicyAssignmentsCreate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [201]:
                return self.on_201(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/{scope}/providers/Microsoft.Authorization/policyAssignments/{policyAssignmentName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "policyAssignmentName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "scope", self.ctx.args.scope,
                    skip_quote=True,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-05-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("identity", AAZIdentityObjectType)
            _builder.set_prop("location", AAZStrType, ".location")
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            identity = _builder.get(".identity")
            if identity is not None:
                identity.set_prop("userAssigned", AAZListType, ".mi_user_assigned", typ_kwargs={"flags": {"action": "create"}})
                identity.set_prop("systemAssigned", AAZStrType, ".mi_system_assigned", typ_kwargs={"flags": {"action": "create"}})

            user_assigned = _builder.get(".identity.userAssigned")
            if user_assigned is not None:
                user_assigned.set_elements(AAZStrType, ".")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("definitionVersion", AAZStrType, ".definition_version")
                properties.set_prop("description", AAZStrType, ".description")
                properties.set_prop("displayName", AAZStrType, ".display_name")
                properties.set_prop("enforcementMode", AAZStrType, ".enforcement_mode")
                properties.set_prop("metadata", AAZDictType, ".metadata")
                properties.set_prop("nonComplianceMessages", AAZListType, ".non_compliance_messages")
                properties.set_prop("notScopes", AAZListType, ".not_scopes")
                properties.set_prop("overrides", AAZListType, ".overrides")
                properties.set_prop("parameters", AAZDictType, ".params")
                properties.set_prop("policyDefinitionId", AAZStrType, ".policy_set_definition")
                properties.set_prop("resourceSelectors", AAZListType, ".resource_selectors")

            metadata = _builder.get(".properties.metadata")
            if metadata is not None:
                metadata.set_elements(AAZAnyType, ".")

            non_compliance_messages = _builder.get(".properties.nonComplianceMessages")
            if non_compliance_messages is not None:
                non_compliance_messages.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.nonComplianceMessages[]")
            if _elements is not None:
                _elements.set_prop("message", AAZStrType, ".message", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("policyDefinitionReferenceId", AAZStrType, ".policy_definition_reference_id")

            not_scopes = _builder.get(".properties.notScopes")
            if not_scopes is not None:
                not_scopes.set_elements(AAZStrType, ".")

            overrides = _builder.get(".properties.overrides")
            if overrides is not None:
                overrides.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.overrides[]")
            if _elements is not None:
                _elements.set_prop("kind", AAZStrType, ".kind")
                _elements.set_prop("selectors", AAZListType, ".selectors")
                _elements.set_prop("value", AAZStrType, ".value")

            selectors = _builder.get(".properties.overrides[].selectors")
            if selectors is not None:
                _CreateHelper._build_schema_selector_create(selectors.set_elements(AAZObjectType, "."))

            parameters = _builder.get(".properties.parameters")
            if parameters is not None:
                parameters.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.parameters{}")
            if _elements is not None:
                _elements.set_prop("value", AAZAnyType, ".value")

            resource_selectors = _builder.get(".properties.resourceSelectors")
            if resource_selectors is not None:
                resource_selectors.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.resourceSelectors[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name")
                _elements.set_prop("selectors", AAZListType, ".selectors")

            selectors = _builder.get(".properties.resourceSelectors[].selectors")
            if selectors is not None:
                _CreateHelper._build_schema_selector_create(selectors.set_elements(AAZObjectType, "."))

            return self.serialize_content(_content_value)

        def on_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_201
            )

        _schema_on_201 = None

        @classmethod
        def _build_schema_on_201(cls):
            if cls._schema_on_201 is not None:
                return cls._schema_on_201

            cls._schema_on_201 = AAZObjectType()

            _schema_on_201 = cls._schema_on_201
            _schema_on_201.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_201.identity = AAZIdentityObjectType()
            _schema_on_201.location = AAZStrType()
            _schema_on_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_201.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_201.type = AAZStrType(
                flags={"read_only": True},
            )

            identity = cls._schema_on_201.identity
            identity.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )
            identity.tenant_id = AAZStrType(
                serialized_name="tenantId",
                flags={"read_only": True},
            )
            identity.type = AAZStrType()
            identity.user_assigned_identities = AAZDictType(
                serialized_name="userAssignedIdentities",
            )

            user_assigned_identities = cls._schema_on_201.identity.user_assigned_identities
            user_assigned_identities.Element = AAZObjectType()

            _element = cls._schema_on_201.identity.user_assigned_identities.Element
            _element.client_id = AAZStrType(
                serialized_name="clientId",
                flags={"read_only": True},
            )
            _element.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )

            properties = cls._schema_on_201.properties
            properties.assignment_type = AAZStrType(
                serialized_name="assignmentType",
            )
            properties.definition_version = AAZStrType(
                serialized_name="definitionVersion",
            )
            properties.description = AAZStrType()
            properties.display_name = AAZStrType(
                serialized_name="displayName",
            )
            properties.effective_definition_version = AAZStrType(
                serialized_name="effectiveDefinitionVersion",
                flags={"read_only": True},
            )
            properties.enforcement_mode = AAZStrType(
                serialized_name="enforcementMode",
            )
            properties.latest_definition_version = AAZStrType(
                serialized_name="latestDefinitionVersion",
                flags={"read_only": True},
            )
            properties.metadata = AAZDictType()
            properties.non_compliance_messages = AAZListType(
                serialized_name="nonComplianceMessages",
            )
            properties.not_scopes = AAZListType(
                serialized_name="notScopes",
            )
            properties.overrides = AAZListType()
            properties.parameters = AAZDictType()
            properties.policy_definition_id = AAZStrType(
                serialized_name="policyDefinitionId",
            )
            properties.resource_selectors = AAZListType(
                serialized_name="resourceSelectors",
            )
            properties.scope = AAZStrType(
                flags={"read_only": True},
            )

            metadata = cls._schema_on_201.properties.metadata
            metadata.Element = AAZAnyType()

            non_compliance_messages = cls._schema_on_201.properties.non_compliance_messages
            non_compliance_messages.Element = AAZObjectType()

            _element = cls._schema_on_201.properties.non_compliance_messages.Element
            _element.message = AAZStrType(
                flags={"required": True},
            )
            _element.policy_definition_reference_id = AAZStrType(
                serialized_name="policyDefinitionReferenceId",
            )

            not_scopes = cls._schema_on_201.properties.not_scopes
            not_scopes.Element = AAZStrType()

            overrides = cls._schema_on_201.properties.overrides
            overrides.Element = AAZObjectType()

            _element = cls._schema_on_201.properties.overrides.Element
            _element.kind = AAZStrType()
            _element.selectors = AAZListType()
            _element.value = AAZStrType()

            selectors = cls._schema_on_201.properties.overrides.Element.selectors
            selectors.Element = AAZObjectType()
            _CreateHelper._build_schema_selector_read(selectors.Element)

            parameters = cls._schema_on_201.properties.parameters
            parameters.Element = AAZObjectType()

            _element = cls._schema_on_201.properties.parameters.Element
            _element.value = AAZAnyType()

            resource_selectors = cls._schema_on_201.properties.resource_selectors
            resource_selectors.Element = AAZObjectType()

            _element = cls._schema_on_201.properties.resource_selectors.Element
            _element.name = AAZStrType()
            _element.selectors = AAZListType()

            selectors = cls._schema_on_201.properties.resource_selectors.Element.selectors
            selectors.Element = AAZObjectType()
            _CreateHelper._build_schema_selector_read(selectors.Element)

            system_data = cls._schema_on_201.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            return cls._schema_on_201


class _CreateHelper:
    """Helper class for Create"""

    @classmethod
    def _build_schema_selector_create(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("in", AAZListType, ".in_")
        _builder.set_prop("kind", AAZStrType, ".kind")
        _builder.set_prop("notIn", AAZListType, ".not_in")

        in_ = _builder.get(".in")
        if in_ is not None:
            in_.set_elements(AAZStrType, ".")

        not_in = _builder.get(".notIn")
        if not_in is not None:
            not_in.set_elements(AAZStrType, ".")

    _schema_selector_read = None

    @classmethod
    def _build_schema_selector_read(cls, _schema):
        if cls._schema_selector_read is not None:
            _schema["in"] = cls._schema_selector_read["in"]
            _schema.kind = cls._schema_selector_read.kind
            _schema.not_in = cls._schema_selector_read.not_in
            return

        cls._schema_selector_read = _schema_selector_read = AAZObjectType()

        selector_read = _schema_selector_read
        selector_read["in"] = AAZListType()
        selector_read.kind = AAZStrType()
        selector_read.not_in = AAZListType(
            serialized_name="notIn",
        )

        in_ = _schema_selector_read["in"]
        in_.Element = AAZStrType()

        not_in = _schema_selector_read.not_in
        not_in.Element = AAZStrType()

        _schema["in"] = cls._schema_selector_read["in"]
        _schema.kind = cls._schema_selector_read.kind
        _schema.not_in = cls._schema_selector_read.not_in


__all__ = ["Create"]
